Package: snapcount
Type: Package
Title: R/Bioconductor Package for interfacing with Snaptron for rapid
        querying of expression counts
Version: 1.23.0
Authors@R: 
    c(person(given = "Rone",
             family = "Charles",
             role = c("aut", "cre"),
             email = "rcharle8@jh.edu"))
Description: snapcount is a client interface to the Snaptron webservices
    which support querying by gene name or genomic region.
    Results include raw expression counts derived from alignment of RNA-seq samples
    and/or various summarized measures of expression across one or more regions/genes
    per-sample (e.g. percent spliced in).
Depends: R (>= 4.0.0)
Imports: R6, httr, rlang, purrr, jsonlite, assertthat, data.table,
        Matrix, magrittr, methods, stringr, stats, IRanges,
        GenomicRanges, SummarizedExperiment
Suggests: BiocManager, bit64, covr, knitcitations, knitr (>= 1.6),
        devtools, BiocStyle (>= 2.5.19), rmarkdown (>= 0.9.5), testthat
        (>= 2.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.1.1
Roxygen: list(markdown = TRUE, roclets = c("rd", "namespace",
        "collate"))
URL: https://github.com/langmead-lab/snapcount
BugReports: https://github.com/langmead-lab/snapcount/issues
biocViews: Coverage, GeneExpression, RNASeq, Sequencing, Software,
        DataImport
VignetteBuilder: knitr
ByteCompile: true
git_url: https://git.bioconductor.org/packages/snapcount
git_branch: devel
git_last_commit: 1af1119
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-11-01 00:18:29 UTC; biocbuild
Author: Rone Charles [aut, cre]
Maintainer: Rone Charles <rcharle8@jh.edu>

# Generated by roxygen2: do not edit by hand

export(Compilation)
export(Coordinates)
export(QueryBuilder)
export(from_url)
export(get_column_filters)
export(get_compilation)
export(get_coordinate_modifier)
export(get_regions)
export(get_row_filters)
export(get_sids)
export(junction_inclusion_ratio)
export(junction_intersection)
export(junction_union)
export(percent_spliced_in)
export(query_exon)
export(query_gene)
export(query_jx)
export(set_column_filters)
export(set_compilation)
export(set_coordinate_modifier)
export(set_regions)
export(set_row_filters)
export(set_sids)
export(shared_sample_counts)
export(tissue_specificity)
export(uri_of_last_successful_request)
import(data.table)
importFrom(GenomicRanges,GRanges)
importFrom(R6,R6Class)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(assertthat,assert_that)
importFrom(magrittr,"%>%")
importFrom(methods,is)
importFrom(rlang,"!!")
importFrom(stats,median)

<!-- README.md is generated from README.Rmd. Please edit that file -->



# snapcount
[![Build Status](https://travis-ci.com/langmead-lab/snapcount.svg?token=vEUBb2QKjox3PdRAssp8&branch=master)](https://travis-ci.com/langmead-lab/snapcount)
[![Coverage Status](https://img.shields.io/codecov/c/github/langmead-lab/snapcount/master.svg)](https://codecov.io/gh/langmead-lab/snapcount?branch=master)

## Overview
`snapcount` is a package for interfacing with Snaptron's REST API.

## Installation


```r
# Install the development version from GitHub:
# install.packages("devtools")
devtools::install_github("langmead-lab/snapcount")
```

## Usage

##### snapcount can be used with either a procedural interface

```r
library(snapcount)

query_jx(compilation = "gtex", regions = "CD99")
#> class: RangedSummarizedExperiment 
#> dim: 3485 9662 
#> metadata(0):
#> assays(1): counts
#> rownames(3485): 28340058 28340273 ... 28352407 28352408
#> rowData names(12): DataSource:Type snaptron_id ... coverage_median
#>   source_dataset_id
#> colnames(9662): 50099 50100 ... 59759 59760
#> colData names(322): rail_id Run ... junction_coverage
#>   junction_avg_coverage
query_jx(compilation = "gtex", regions = "CD99", range_filters = samples_count == 10)
#> class: RangedSummarizedExperiment 
#> dim: 25 226 
#> metadata(0):
#> assays(1): counts
#> rownames(25): 28342638 28343346 ... 28352394 28352402
#> rowData names(12): DataSource:Type snaptron_id ... coverage_median
#>   source_dataset_id
#> colnames(226): 50117 50145 ... 59629 59713
#> colData names(322): rail_id Run ... junction_coverage
#>   junction_avg_coverage
```

##### Or using the query-builder class

```r
sb <- SnaptronQueryBuilder$new()
sb$compilation("gtex")$regions("CD99")$query_jx()
#> class: RangedSummarizedExperiment 
#> dim: 3485 9662 
#> metadata(0):
#> assays(1): counts
#> rownames(3485): 28340058 28340273 ... 28352407 28352408
#> rowData names(12): DataSource:Type snaptron_id ... coverage_median
#>   source_dataset_id
#> colnames(9662): 50099 50100 ... 59759 59760
#> colData names(322): rail_id Run ... junction_coverage
#>   junction_avg_coverage

# sb$from_url("http://snaptron.cs.jhu.edu/gtex/bases?regions=BRCA1&sids=50099,50102,50113")$query_gene()
```

## Reference
For more information on Snaptron please visit: http://snaptron.cs.jhu.edu/index.html.

# nocov start
pkg_globals <- new.env(parent = emptyenv())

.onLoad <- function(...) {
    snaptron_host <- "http://snaptron.cs.jhu.edu"

    if (!is.null(host <- getOption("snapcount.host"))) {
        snaptron_host <- host
    }
    if (!is.null(port <- getOption("snapcount.port"))) {
        if (is.wholenumber(port) && port != 80) {
            snaptron_host <- paste(snaptron_host, port, sep = ":")
        }
    }
    snaptron_host <- paste0(snaptron_host, "/")
    assign("snaptron_host", snaptron_host, envir = pkg_globals)
    json_data <-
        paste(pkg_globals$snaptron_host, "snaptron", "registry", sep = "/") %>%
        httr::GET()
    if (httr::http_error(json_data)) {
        return()
    }
    start <- 8
    end <- length(json_data$content)
    registry <- json_data$content[start:end] %>%
        rawToChar() %>% jsonlite::fromJSON()
    compilation_names <- names(registry) %>% as.list()

    if (length(compilation_names) == 0) {
        return()
    }
    assign("registry", registry, envir = pkg_globals)
    assign("metadata", list(), envir = pkg_globals)
    if (!is.null(compilation_names)) {
        compilations <- do.call(enum, compilation_names)
        assign("Compilation", compilations, parent.env(environment()))
    }
}

#nocov end

#' snapcount: an R package for interfacing with Snaptron
#'
#' snapcount is a client interface to the Snaptron webservice which supports
#' querying by gene name or genomic region.
#'
#' Results include raw expression counts derived from alignment of RNA-seq
#' samples and/or various summarized measures of expression across one or more
#' regions/genes per-sample (e.g. percent spliced in).
#'
#' To learn more about snapcount, check out the vignette:
#' \code{browseVignettes(package = "snapcount")}
#'
#' @section Package options:
#' \describe{
#' \item{\code{snapcount.host}}{Change the host that snapcount uses when
#'   connecting to Snaptron. Default: \code{snaptron.cs.jhu.edu}}
#' \item{\code{snapcount.port}}{Change the port that snapcount uses when
#'   connecting to Snaptron. Default: \code{80}}
#' }
#'
#' @import data.table
#' @importFrom assertthat assert_that
#' @importFrom magrittr %>%
#' @importFrom methods is
#' @importFrom rlang !!
#' @importFrom stats median
#' @importFrom GenomicRanges GRanges
#' @importFrom R6 R6Class
#' @importFrom SummarizedExperiment SummarizedExperiment
"_PACKAGE"

is_hugo_gene <- function(str) {
    assert_that(is.character(str))
    stringr::str_detect(str, "^[A-Za-z0-9_]+@?$")
}

is_genes_or_intervals <- function(strings) {
    bools <-
        Map(function(s) is_hugo_gene(s) || is_chromosome_interval(s), strings)
    Reduce(`&&`, bools, TRUE)
}

is_chromosome_interval <- function(str) {
    assert_that(is.character(str))
    stringr::str_detect(str, "chr([1-9]|1[0-9]|2[0-2]|[XYM]):\\d+-\\d+")
}

is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
    assert_that(is.numeric(x))
    all(x >= 0 & (abs(x - round(x)) < tol))
}

is_query_builder <- function(object) {
    is(object, "SnaptronQueryBuilder") && is(object, "R6")
}

is_list_of_query_builders <- function(list) {
    assert_that(is.list(list))
    purrr::reduce(lapply(list, is_query_builder), all)
}

is_list_of_query_builder_groups <- function(list_of_groups) {
    assert_that(is.list(list_of_groups))
    purrr::reduce(lapply(list_of_groups, is_list_of_query_builders), all)
}

# nocov start
enum <- function(...) {
    variants <- rlang::ensyms(...)
    values <- purrr::map(variants, rlang::as_string)

    res <- stats::setNames(values, values)
    res <- as.environment(as.list(res))

    lockEnvironment(res, bindings = TRUE)

    res
}
#' Enum for Snaptron Coordinate modifiers
#'
#' @field Exact Return junctions whose start and end
#'   coordinates match the boundaries of the
#'   region requested.
#' @field Within Return junctions whose start and end
#'   coordinates are within the boundaries of the
#'   region requested.
#' @field StartIsExactOrWithin Return junctions whose
#'   start coordinate matches, or is within, the
#'   boundaries of the region requested.
#' @field EndIsExactOrWithin Return junctions whose
#'   end coordinate matches, or is within, the
#'   boundaries of the region requested.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' qb <- set_coordinate_modifier(qb, Coordinates$Exact)
#' qb
#' @export
Coordinates <- enum(Exact, Within, StartIsExactOrWithin, EndIsExactOrWithin)

#' Enum for Snaptron compilations
#'
#' The variants for this enum will be populated dynamically after the
#' package has been loaded. If the package cannot connect to the internet
#' the variants will default to:
#'
#' \itemize{
#'   \item{gtex}
#'   \item{tcga}
#'   \item{srav2}
#'   \item{sra}
#' }
#'
#' @seealso \url{http://snaptron.cs.jhu.edu/data.html} for more information
#'   about Snaptron compilations.
#'
#' @examples
#' qb <- QueryBuilder(compilation = Compilation$gtex, regions = "KCNIP4")
#' query_jx(qb)
#' @export
Compilation <- enum(gtex, tcga, srav2, sra)

#nocov end

#' Get the union of junctions from 2 or more compilations
#' which are on the same reference
#'
#' This function queries 2 or more compilations which are on the same
#' reference version (e.g. hg38) and merges the resulting junctions
#' into a single output table, unioning the sample coverage columns
#' and the snaptron_id (jx ID) columns (the latter delimiter will
#' be ":").  All sample IDs will be disjoint between compilations.
#'
#' Union is based on the following fields (combined into a comparison key):
#' \itemize{
#'   \item group
#'   \item chromosome
#'   \item start
#'   \item end
#'   \item strand
#' }
#'
#' The goal is to have a single list of junctions where every junction
#' occurs in at least one compilation and if a junction occurs in > 1
#' compilations it still only has a single row representing all the
#' samples across compilations that it appears in.
#' Sample aggregate statistics will be recalculated for junctions which
#' are merged across *all* samples from all compilations:
#'
#'
#' \itemize{
#'   \item sample_count
#'   \item coverage_sum
#'   \item coverage_avg
#'   \item coverage_median
#' }
#'
#'
#' @param ... One or more QueryBuilder objects
#' @return A RangedSummarizedExperiment of junctions appearing in at least
#' one compilation
#' @seealso [junction_intersection()]
#'
#' @examples
#' # Using query builder wrappers
#' sb1 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879786-1879786")
#' sb1 <- set_coordinate_modifier(sb1, Coordinates$EndIsExactOrWithin)
#' sb1 <- set_row_filters(sb1, strand == "-")
#'
#' sb2 <- QueryBuilder(compilation = "tcga", regions = "chr1:1879786-1879786")
#' sb2 <- set_coordinate_modifier(sb2, Coordinates$EndIsExactOrWithin)
#' sb2 <- set_row_filters(sb2, strand == "-")
#'
#' junction_union(sb1, sb2)
#' @export
junction_union <- function(...) {
    assert_that(
        is_list_of_query_builders(list(...)),
        msg = "junction_union expects 1 or more QueryBuilder objects")
    merge_compilations(..., all = TRUE)
}

#' Get the intersection of junctions from 2 or more compilations
#' which are on the same reference
#'
#' This function operates similar to the `junction_union()` function, i.e
#' it is cross compilation and merging of the same junction from multiple
#' compilations will be handled exactly the same way. But instead
#' of every junction which appears in at least one compilation, only
#' the junctions which appear in *every* compilation will be returned.
#'
#' @param ... One or more QueryBuilder objects
#' @return A RangedSummarizedExperiment of junctions common across compilations
#' @seealso [junction_union()]
#'
#' @examples
#' # Using query builder wrappers
#' sb1 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879786-1879786")
#' sb1 <- set_coordinate_modifier(sb1, Coordinates$EndIsExactOrWithin)
#' sb1 <- set_row_filters(sb1, strand == "-")
#'
#' sb2 <- QueryBuilder(compilation = "tcga", regions = "chr1:1879786-1879786")
#' sb2 <- set_coordinate_modifier(sb2, Coordinates$EndIsExactOrWithin)
#' sb2 <- set_row_filters(sb2, strand == "-")
#'
#' junction_intersection(sb1, sb2)
#' @export
junction_intersection <- function(...) {
    assert_that(
        is_list_of_query_builders(list(...)),
        msg = paste(
            "junction_intersection expects 1 or more",
            "QueryBuilder objects"
        )
    )
    merge_compilations(..., all = FALSE)
}

merge_compilations <- function(..., all) {
    compilations <- lapply(list(...), function(sb) { sb$compilation() })
    query_data_list <- lapply(list(...), function(sb) {
        sb$query_jx(return_rse = FALSE)
    })
    metadata_list <- lapply(unique(compilations), function(compilation) {
        get_compilation_metadata(compilation)
    })

    if (length(query_data_list) == 1) {
        return(rse(query_data_list[[1]], metadata_list[[1]]))
    }

    for (i in seq_along(query_data_list)) {
        if (i == 1) {
            merged_query_data <- query_data_list[[1]]
        } else {
            merged_query_data <-
                merge(
                    merged_query_data, query_data_list[[i]], all = all,
                    by = c("chromosome", "start", "end", "strand")
                ) %>% finalize_merge(col_names = names(query_data_list[[1]]))
        }
    }

    for (i in seq_along(metadata_list)) {
        if (i == 1) {
            merged_metadata <- metadata_list[[1]]
        } else {
            merged_metadata <-
                merge(
                    merged_metadata, metadata_list[[i]],
                    by = "rail_id", all = TRUE
                )
        }
    }

    rse(merged_query_data, merged_metadata)
}

finalize_merge <- function(dt, col_names) {
    dt$samples <- str_cat(dt$samples.x, dt$samples.y)
    dt$snaptron_id <- str_cat(dt$snaptron_id.x, dt$snaptron_id.y, sep = ",")
    dt$`DataSource:Type` <-
        str_cat(dt$`DataSource:Type.x`, dt$`DataSource:Type.y`, sep = ",")
    dt$source_dataset_id <-
        str_cat(dt$source_dataset_id.x, dt$source_dataset_id.y, sep = ",")
    dt$coverage_sum <-
        purrr::map2_int(dt$coverage_sum.x, dt$coverage_sum.y, sum, na.rm = TRUE)
    dt$coverage_avg <- dt$coverage_sum / dt$samples_count
    dt$coverage_median <-
        stringr::str_extract_all(dt$samples, "\\b\\d+\\b") %>%
        lapply(calculate_coverage_median) %>% unlist()
    dt$length <- choose_non_na(dt$length.x, dt$length.y)
    dt$left_motif <- choose_non_na(dt$left_motif.x, dt$left_motif.y)
    dt$right_motif <- choose_non_na(dt$right_motif.x, dt$right_motif.y)
    dt$annotated <- choose_non_na(dt$annotated.x, dt$annotated.y)
    dt$left_annotated <-
        choose_non_na(dt$left_annotated.x, dt$left_annotated.y)
    dt$right_annotated <-
        choose_non_na(dt$right_annotated.x, dt$right_annotated.y)
    dt$samples_count <-
        purrr::map2_int(
                   dt$samples_count.x,
                   dt$samples_count.y,
                   sum, na.rm = TRUE
               )

    dt[, col_names, with = FALSE]
}

str_cat <- function(..., sep = "") {
    strings <- lapply(list(...), stringr::str_replace_na, replacement = "")
    paste(strings[[1]], strings[[2]], sep = sep)
}

choose_non_na <- function(x, y) {
    ifelse(is.na(x), y, x)
}

calculate_coverage_median <- function(samples) {
    samples[c(FALSE, TRUE)] %>% as.numeric() %>% median()
}

#' Relative measure of splice variant usage similar to PSI that allows
#' for 2 arbitrarily defined groups of junctions (not limited to
#' cassette exons).
#'
#' Calculates a coverage summary statistic per sample of the normalized
#' coverage difference between two sets of separate junctions defined
#' by at least two basic queries and organized into two groups.
#'
#' The summary statistic is as follows:
#' If the coverage of the first group is "A" and the second is "B":
#'
#' `JIR(A,B)=(A - B) / (A+B+1)`
#'
#' This is calculated for every sample that occurs in one or the other
#' (or both) groups results.
#'
#' @param group1,group2 Each group is a list of 1 or more
#'     QueryBuilder objects
#' @param group_names Optional vector of strings representing the
#'     group names
#' @return A DataFrame of samples, with their JIR score and metadata,
#'     which had > 0 coverage in at least one resulting row in at
#'     least one of the groups
#'
#' @examples
# Using query builder wappers
#' sb1 <- QueryBuilder(compilation = "srav2", regions = "chr2:29446395-30142858")
#' sb1 <- set_coordinate_modifier(sb1, Coordinates$Within)
#' sb1 <- set_row_filters(sb1, strand == "-")
#'
#' sb2 <- QueryBuilder(compilation = "srav2", regions = "chr2:29416789-29446394")
#' sb2 <- set_coordinate_modifier(sb2, Coordinates$Within)
#' sb2 <- set_row_filters(sb2, strand == "-")
#'
#' junction_inclusion_ratio(list(sb1), list(sb2))
#' @export
junction_inclusion_ratio <- function(group1, group2, group_names = NULL) {
    assert_that(is_list_of_query_builders(group1),
                is_list_of_query_builders(group2))

    query_results <- run_queries(group1, group2)
    if (is.null(query_results[[1]])) {
        stop("Unable to calculate JIR: group1 returned no results")
    }
    if (is.null(query_results[[2]])) {
        stop("Unable to calculate JIR: group2 returned no results")
    }

    jir <- merge(query_results[[1]], query_results[[2]],
                 by = "sample_id", all = TRUE) %>% replace_na(0)

    ## jir[, jir := calc_jir(coverage.y, coverage.x)]
    jir$jir <- calc_jir(jir$coverage.y, jir$coverage.x)

    if (is.null(group_names)) {
        group_names <- c("group1_coverage", "group2_coverage")
    } else {
        group_names <- paste(group_names, "coverage", sep = "_")
    }

    data.table::setnames(jir, old = "coverage.x", new = group_names[[1]])
    data.table::setnames(jir, old = "coverage.y", new = group_names[[2]])

    jir[order(jir, decreasing = TRUE)]
}

calc_jir <- function(a, b) {
    (a - b)/(a + b + 1)
}

#' Relative measure of splice variant usage, limited currently to
#' cassette exon splice variants
#'
#' Similar to the JIR, this calculates Percent Spliced In (PSI) statistics
#' for the definition of 2 different groups: inclusion and exclusion.
#' Currently this function only supports the cassette exon use case.
#'
#' Inclusion typically defines 2 basic queries, one for the junction
#' preceding the cassette exon, and the second for the junction following
#' the cassette exon.  The exclusion group contains one basic query
#' which defines the junction which skips the cassette exon.
#'
#' The PSI itself is implemented as:
#'
#'
#' \code{PSI(inclusion1, inclusion2, exclusion) =
#'   mean(inclusion1, inclusion2) / (mean(inclusion1, inclusion2) + exclusion)}
#'
#' where each term denotes the coverage of junctions that resulted
#' from the basic queries in that group in the current sample.
#'
#' @param inclusion_group1,inclusion_group2,exclusion_group Where each
#'     is a list of 1 or more QueryBuilder objects
#' @param min_count minimum total count (denominator) required to not
#'     be assigned -1
#' @param group_names Optional vector of strings representing the
#'     group names
#' @return A DataFrame of samples, with their PSI score and metadata,
#'     which had > 0 coverage in at least one resulting row in at
#'     least one of the groups
#'
#' @examples
# Using query builder wrappers
#' in1 <- QueryBuilder(compilation = "srav2", regions = "chr1:94468008-94472172")
#' in1 <- set_coordinate_modifier(in1, Coordinates$Exact)
#' in1 <- set_row_filters(in1, strand == "+")
#'
#' in2 <- QueryBuilder(compilation = "srav2", regions = "chr1:94468008-94472172")
#' in2 <- set_coordinate_modifier(in2, Coordinates$Exact)
#' in2 <- set_row_filters(in2, strand == "+")
#'
#' ex <- QueryBuilder(compilation = "srav2", regions = "chr1:94468008-94475142")
#' ex <- set_coordinate_modifier(ex, Coordinates$Exact)
#' ex <- set_row_filters(ex, strand == "+")
#'
#' percent_spliced_in(list(in1), list(in2), list(ex))
#' @export
percent_spliced_in <- function(inclusion_group1, inclusion_group2,
                               exclusion_group, min_count = 20,
                               group_names = NULL)
{
    query_results <-
        run_queries(inclusion_group1, inclusion_group2, exclusion_group)
    if (is.null(query_results[[1]])) {
        stop("Unable to calculate PSI: inclusion_group1 returned no results")
    }
    if (is.null(query_results[[2]])) {
        stop("Unable to calculate PSI: inclusion_group2 returned no results")
    }
    if (is.null(query_results[[3]])) {
        stop("Unable to calculate PSI: exclusion_group returned no results")
    }

    psi <-
        merge(query_results[[1]], query_results[[2]],
                 by = "sample_id", all = TRUE) %>%
        merge(query_results[[3]], by = "sample_id", all = TRUE) %>%
        replace_na(0)

    ## psi[, psi := calc_psi(coverage.x, coverage.y, coverage, min_count)]
    psi$psi <- calc_psi(psi$coverage.x, psi$coverage.y, psi$coverage, min_count)

    if (is.null(group_names)) {
        group_names <- c(
            "inclusion_group1_coverage",
            "inclusion_group2_coverage",
            "exclusion_group_coverage"
        )
    } else {
        group_names <- paste(group_names, "coverage", sep = "_")
    }

    data.table::setnames(psi, old = "coverage.x", new = group_names[[1]])
    data.table::setnames(psi, old = "coverage.y", new = group_names[[2]])
    data.table::setnames(psi, old = "coverage", new = group_names[[3]])

    psi
}

calc_psi <- function(inclusion1, inclusion2, exclusion, min_count) {
    mean_inclusion <- (inclusion1 + inclusion2) / 2.
    total <- mean_inclusion + exclusion
    psi <- mean_inclusion / total

    ifelse(inclusion1 == 0 | inclusion2 == 0 | total < min_count, -1, psi)
}

#' Tissue Specificity (TS): produces a list of samples with their tissues
#' marked which either contain queried junctions (1) or not (0); can be used
#' as input to significance testing methods such as Kruskal-Wallis to look for
#' tissue enrichment (currently only works for the GTEx compilation).
#'
#' Lists the number of samples labeled with a specific tissue type.

#' Samples are filtered for ones which have junctions across all the
#' user-specified groups. That is, if a sample only appears in the results of
#' some of the groups (from their basic queries) it will be assigned a 0,
#' otherwise if it is in all of the groups' results it will be assigned a 1.
#' This is similar to the SSC high level query type, but doesn't sum the
#' coverage.
#'
#' The samples are then grouped by their tissue type (e.g. Brain).
#' This is useful for determining if there's an enrichment for a specific
#' tissue in the set of junctions queried.  Results from this can be fed to a
#' statistical test, such as the Kruskal-wallis non-parametric rank test.

#' This query is limited to GTEx only, due to the fact that GTEx is one of the
#' few compilations that has consistent and complete tissue metadata.
#'
#' @param ... One or more QueryBuilder objects
#' @param group_names Optional vector of strings representing the
#'   group names
#' @return A DataFrame of all samples in the compilation with either a
#'   0 or 1 indicating their occurrence and shared status (if > 1
#'   group passed in).  Occurrence here is if the sample has at least
#'   one result with > 0 coverage, and further, if > 1 group is passed
#'   in, then if it occurs in the results of all groups.  Also includes the
#'   sample tissue type and sample_id.
#'
#' @examples
# Using query builder wrappers
#' in1 <- QueryBuilder(compilation = "gtex", regions = "chr4:20763023-20763023")
#' in1 <- set_coordinate_modifier(in1, Coordinates$EndIsExactOrWithin)
#' in1 <- set_row_filters(in1, strand == "-")
#'
#' in2 <- QueryBuilder(compilation = "gtex", regions = "chr4:20763098-20763098")
#' in2 <- set_coordinate_modifier(in2, Coordinates$StartIsExactOrWithin)
#' in2 <- set_row_filters(in2, strand == "-")
#'
#' tissue_specificity(list(in1, in2))
#' @export
tissue_specificity <- function(..., group_names = NULL) {
    list_of_groups <- list(...)
    assert_that(is_list_of_query_builder_groups(list_of_groups))

    num_groups <- length(list_of_groups)

    if (is.null(group_names)) {
        group_names <- paste0("g", seq_along(num_groups))
    }

    dfs <- lapply(seq_along(num_groups), function(i) {
        g <- list_of_groups[[i]]
        name <- group_names[[i]]
        if (length(g) == 1) {
            tissue_specificity_per_group(g[[1]], g[[1]], name)
        } else{
            tissue_specificity_per_group(g[[1]], g[[2]], name)
        }
    })

    do.call(rbind, dfs)
}

tissue_specificity_per_group <- function(group1, group2, group_name) {
    if (is_query_builder(group1)) {
        group1 <- list(group1)
    }

    if (is_query_builder(group2)) {
        group2 <- list(group2)
    }

    stopifnot(is.list(group1), is.list(group2))
    query_data <- run_queries(group1, group2, summarize = FALSE)
    if (is.null(query_data[[1]])) {
        stop("Unable to calculate TS: group1 returned no results")
    }
    if (is.null(query_data[[2]])) {
        stop("Unable to calculate TS: group2 returned no results")
    }

    merged_data <- merge(
        query_data[[1]], query_data[[2]],
        by = "sample_id", all = TRUE
    ) %>% replace_na(0)
    merged_data$shared <- shared(merged_data$coverage.x, merged_data$coverage.y)
    merged_data <- merged_data[, !c("coverage.x", "coverage.y")]

    metadata <- get_compilation_metadata(group1[[1]]$compilation())
    metadata <- metadata[, c("rail_id", "SMTS")]

    ts <- merge(merged_data, metadata, by.x = "sample_id",
                by.y = "rail_id", all = TRUE)
    ts <- replace_na(ts, 0, colnames = setdiff(names(ts), "SMTS"))

    ts$group <- rep(group_name, nrow(ts))
    data.table::setnames(ts, old = "SMTS", new = "tissue")

    unique(ts)
}

shared <- function(cov1, cov2) {
    cov1 != 0 & cov2 != 0
}

#' Shared Sample Count (SSC): counts total number of samples in which 2
#' different junctions both occur in.
#'
#' This produces a list of user-specified groups and the read coverage
#' of the junctions in all the samples which were shared across all
#' the basic queries occurring in each group.
#'
#' Example: User defines a single group of junctions "GroupA" made up of 2
#' separate regions (two basic queries).
#'
#' An SSC query will return a single line for GroupA which will have
#' the total number of samples which had at least one junction which
#' was returned from both basic queries. It will also report a summary
#' statistic of the total number of groups which had one or more
#' samples that were shared across the basic queries, in this case it
#' would be 1.  Also, it will report the number of groups which had at
#' least one shared sample and which had matching junctions (from the
#' query) which were fully annotated.
#'
#' This function can be used to determine how much cross-sample
#' support there is for a particular junction configuration (typically
#' a cassette exon).
#'
#' @param ... One or more lists of QueryBuilder objects
#' @param group_names Optional vector of character strings
#'   representing group names
#' @return A DataFrame of results based on the list of groups passed
#'   in via "group_names".  Each group is reported with the # of
#'   unique samples which occurred in all of its defined set of
#'   related basic queries (e.g. two inclusion basic queries in a
#'   cassette exon scenario).
#'
#' @examples
# Using query builder wrappers
#' g1 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879786-1879786")
#' g1 <- set_coordinate_modifier(g1, Coordinates$EndIsExactOrWithin)
#' g1 <- set_row_filters(g1, strand == "-")
#'
#' g2 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879903-1879903")
#' g2 <- set_coordinate_modifier(g2, Coordinates$StartIsExactOrWithin)
#' g2 <- set_row_filters(g2, strand == "-")
#'
#' ssc<-shared_sample_counts(list(g1, g2))
#' @export
shared_sample_counts <- function(..., group_names = NULL) {
    list_of_groups <- list(...)
    assert_that(is_list_of_query_builder_groups(list_of_groups))

    counts <- lapply(list_of_groups, function(g) {
        shared_sample_count(g[[1]], g[[2]])
    }) %>% unlist()

    if (is.null(group_names)) {
        group_names <- paste0("g", seq_along(list_of_groups))
    }

    data.table(group = group_names, counts = counts)
}

shared_sample_count <- function(group1, group2) {
    if (is_query_builder(group1)) {
        group1 <- list(group1)
    }

    if (is_query_builder(group2)) {
        group2 <- list(group2)
    }

    stopifnot(is.list(group1), is.list(group2))

    query_results <- run_queries(group1, group2)
    if (is.null(query_results[[1]])) {
        stop("Unable to calculate SSC: group1 returned no results")
    }
    if (is.null(query_results[[2]])) {
        stop("Unable to calculate SSC: group2 returned no results")
    }
    intersect(query_results[[1]]$sample_id, query_results[[2]]$sample_id) %>%
        length()
}

run_queries <- function(..., summarize = TRUE) {
    lapply(list(...), count_samples, summarize = summarize)
}

count_samples <- function(group, summarize = TRUE) {
    dfs <- lapply(group, function(sb) {
        query_data <- sb$query_jx(return_rse = FALSE)
        if (is.null(query_data)) {
            return()
        }
        dt <- data.table(sample = extract_samples(query_data))
        dt <- dt[, c("sample_id", "coverage") := tstrsplit(dt$sample, ":")]
        dt$coverage <- as.numeric(dt$coverage)
        dt$sample_id <- as.numeric(dt$sample_id)

        dt[, c("sample_id", "coverage")]
    })

    res <- do.call(rbind, dfs)

    if (summarize && !is.null(res)) {
        ## res[, .(coverage = sum(coverage)), by = .(sample_id)]
        res[, lapply(.SD, sum), by = c("sample_id"), .SDcols = c("coverage")]
    } else {
        res
    }
}

replace_na <- function(dt, replacement, colnames = NULL) {
    if (!is.null(colnames)) {
        for (name in colnames) {
            set(dt, which(is.na(dt[[name]])), name, replacement)
        }
    } else {
        for (i in seq_along(dt)) {
            set(dt, which(is.na(dt[[i]])), i, replacement)
        }
    }

    dt
}

context("Test high level functions")

get_full_path_name <- function(file_name) {
    full_path_name <-
        system.file("testdata", file_name, package = "snapcount")
}

test_that("junction union", {
    sb1 <- SnaptronQueryBuilder$new()
    sb1$compilation("encode1159")
    sb1$regions("chr1:1879786-1879786")
    sb1$coordinate_modifier(Coordinates$EndIsExactOrWithin)
    sb1$row_filters(strand == "-")

    sb2 <- sb1$clone(deep = TRUE)
    sb2$compilation("rpc")

    expected <- readRDS(file = get_full_path_name("test_junction_union_output.rds"))
    result <- junction_union(sb1, sb2)

    expect_true(all(expected == result))
})

test_that("shared sample count", {
    ## group 1
    sb1 <- SnaptronQueryBuilder$new()
    sb1$compilation("gtex")
    sb1$regions("chr1:1879786-1879786")
    sb1$coordinate_modifier(Coordinates$EndIsExactOrWithin)
    sb1$row_filters(strand == "-")

    sb2 <- sb1$clone(deep = TRUE)
    sb2$regions("chr1:1879903-1879903")
    sb2$coordinate_modifier(Coordinates$StartIsExactOrWithin)
    group1 <- list(sb1, sb2)

    ## group 2
    sb3 <- sb1$clone(deep = TRUE)
    sb3$regions("chr1:9664595-9664595")
    sb3$row_filters(strand == "+")

    sb4 <- sb2$clone(deep = TRUE)
    sb4$regions("chr1:9664759-9664759")
    sb4$row_filters(strand == "+")
    group2 <- list(sb3, sb4)

    ## group 3
    sb5 <- sb1$clone(deep = TRUE)
    sb5$regions("chr6:32831148-32831148")

    sb6 <- sb2$clone(deep = TRUE)
    sb6$regions("chr6:32831182-32831182")
    group3 <- list(sb5, sb6)

    result <- shared_sample_counts(group1, group2, group3)
    expected <- readRDS(file = get_full_path_name("test_ssc_output.rds"))

    expect_equal(expected, result)

    sb0 <- sb5$clone(deep = TRUE)$coordinate_modifier(Coordinates$Within)
    suppressWarnings(
        expect_error(shared_sample_counts(group1, group2, list(sb0, sb6)),
                     "group1 returned no results"))

    suppressWarnings(
        expect_error(shared_sample_counts(group1, group2, list(sb5, sb0)),
                     "group2 returned no results"))
})

test_that("junction inclusion ratio", {
    sb1 <- SnaptronQueryBuilder$new()
    sb1$compilation("srav2")
    sb1$regions("chr2:29446395-30142858")
    sb1$coordinate_modifier(Coordinates$Within)
    sb1$row_filters(strand == "-")

    sb2 <- sb1$clone(deep = TRUE)
    sb2$regions("chr2:29416789-29446394")

    result <- junction_inclusion_ratio(list(sb1), list(sb2))
    expected <- readRDS(file = get_full_path_name("test_jir_output.rds"))

    expect_equal(expected, result)

    sb0 <- sb1$clone(deep = TRUE)$regions("CD99")$coordinate_modifier(Coordinates$Exact)
    suppressWarnings(
        expect_error(junction_inclusion_ratio(list(sb0), list(sb2)),
                     "group1 returned no results"))

    suppressWarnings(
        expect_error(junction_inclusion_ratio(list(sb1), list(sb0)),
                     "group2 returned no results"))
})

test_that("percent spliced in", {
    ## inclusion group 1
    sb1 <- SnaptronQueryBuilder$new()
    sb1$compilation("srav2")
    sb1$regions("chr1:94468008-94472172")
    sb1$coordinate_modifier(Coordinates$Exact)
    sb1$row_filters(strand == "+")

    ## inclusion group 2
    sb2 <- sb1$clone(deep = TRUE)
    sb2$regions("chr1:94472243-94475142")

    ## exclusion group
    sb3 <- sb1$clone(deep = TRUE)
    sb3$regions("chr1:94468008-94475142")

    result <- percent_spliced_in(list(sb1), list(sb2), list(sb3), min_count = 1)
    expected <- readRDS(file = get_full_path_name("test_psi_output.rds"))

    expect_equal(expected, result)

    sb0 <- sb1$clone(deep = TRUE)$regions("CD99")
    suppressWarnings(
        expect_error(percent_spliced_in(list(sb0), list(sb2), list(sb3)),
                     "inclusion_group1 returned no results"))

    suppressWarnings(
        expect_error(percent_spliced_in(list(sb1), list(sb0), list(sb3)),
                     "inclusion_group2 returned no results"))

    suppressWarnings(
        expect_error(percent_spliced_in(list(sb1), list(sb2), list(sb0)),
                 "exclusion_group returned no results"))
})

test_that("tissue specificity", {
    sb1 <- SnaptronQueryBuilder$new()
    sb1$compilation("gtex")
    sb1$regions("chr4:20763023-20763023")
    sb1$coordinate_modifier(Coordinates$EndIsExactOrWithin)
    sb1$row_filters(strand == "-")

    sb2 <- sb1$clone(deep = TRUE)
    sb2$regions("chr4:20763098-20763098")
    sb2$coordinate_modifier(Coordinates$StartIsExactOrWithin)

    result <- tissue_specificity(list(sb1, sb2))
    expected <- readRDS(file = get_full_path_name("test_ts_output.rds"))

    expect_equal(expected, result)

    sb0 <- sb1$clone(deep = TRUE)$coordinate_modifier(Coordinates$Within)
    suppressWarnings(
        expect_error(tissue_specificity(list(sb0, sb2)),
                     "group1 returned no results"))

    suppressWarnings(
        expect_error(tissue_specificity(list(sb1, sb0)),
                 "group2 returned no results"))
})

#check higher level functions in Snapcount

library(snapcount)

######Merge testing
sb <- SnaptronQueryBuilder$new()

urls1=list("http://snaptron.cs.jhu.edu/encode1159/snaptron?regions=chr1:1879786-1879786&either=2&rfilter=strand:-",
           "http://snaptron.cs.jhu.edu/rpc/snaptron?regions=chr1:1879786-1879786&either=2&rfilter=strand:-")

len<-length(urls1)
sbs1<-lapply(urls1, function(g) { sb$from_url(g)$clone(deep=TRUE) })

check_junction_union_output<-junction_union(sbs1[[1]],sbs1[[2]])

test_junction_union_output<-readRDS(file="test_junction_union_output.rds")
all.equal(test_junction_union_output, check_junction_union_output)


######Intersection testing
sb <- SnaptronQueryBuilder$new()

urls1=list("http://snaptron.cs.jhu.edu/encode1159/snaptron?regions=chr1:1879786-1879786&either=2&rfilter=strand:-",
           "http://snaptron.cs.jhu.edu/rpc/snaptron?regions=chr1:1879786-1879786&either=2&rfilter=strand:-")

len<-length(urls1)
sbs1<-lapply(urls1, function(g) { sb$from_url(g)$clone(deep=TRUE) })

check_junction_intersection_output<-junction_intersection(sbs1[[1]],sbs1[[2]])

test_junction_intersection_output<-readRDS(file="test_junction_intersection_output.rds")
all.equal(test_junction_intersection_output, check_junction_intersection_output)



####SSC testing
#Shared Sample Count (SSC) high level function fails
sb <- SnaptronQueryBuilder$new()
urls1=list("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr1:1879786-1879786&either=2&rfilter=strand:-",
           "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr1:9664595-9664595&either=2&rfilter=strand:+",
           "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr6:32831148-32831148&either=2&rfilter=strand:-")
urls2=c("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr1:1879903-1879903&either=1&rfilter=strand:-",
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr1:9664759-9664759&either=1&rfilter=strand:+",
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr6:32831182-32831182&either=1&rfilter=strand:-")

len<-length(urls1)
sbs1<-lapply(urls1, function(g) { sb$from_url(g)$clone(deep=TRUE) })
sbs2<-lapply(urls2, function(g) { sb$from_url(g)$clone(deep=TRUE) })
ssc_inputs<-lapply(1:length(sbs1), function(g) { list(sbs1[[g]], sbs2[[g]])})
check_ssc_output <- shared_sample_counts(ssc_inputs[[1]], ssc_inputs[[2]], ssc_inputs[[3]])

test_ssc_output<-readRDS(file="test_ssc_output.rds")
all.equal(test_ssc_output, check_ssc_output)


########JIR testing
sb1 <- SnaptronQueryBuilder$new()
sb1<-sb1$from_url("http://snaptron.cs.jhu.edu/srav2/snaptron?regions=chr2:29446395-30142858&contains=1&rfilter=strand:-")
sb2 <- SnaptronQueryBuilder$new()
sb2<-sb2$from_url("http://snaptron.cs.jhu.edu/srav2/snaptron?regions=chr2:29416789-29446394&contains=1&rfilter=strand:-")
check_jir_output<-junction_inclusion_ratio(list(sb1),list(sb2))

#load(file="test_jir_output.rds")
test_jir_output<-readRDS(file="test_jir_output.rds")
all.equal(test_jir_output, check_jir_output)


######PSI testing
inclusion_group1 <- SnaptronQueryBuilder$new()
inclusion_group1 <- inclusion_group1$from_url("http://snaptron.cs.jhu.edu/srav2/snaptron?regions=chr1:94468008-94472172&exact=1&rfilter=strand:+")
inclusion_group2 <- SnaptronQueryBuilder$new()
inclusion_group2 <- inclusion_group2$from_url("http://snaptron.cs.jhu.edu/srav2/snaptron?regions=chr1:94472243-94475142&exact=1&rfilter=strand:+")
exclusion_group <- SnaptronQueryBuilder$new()
exclusion_group <- exclusion_group$from_url("http://snaptron.cs.jhu.edu/srav2/snaptron?regions=chr1:94468008-94475142&exact=1&rfilter=strand:+")

check_psi_output<-percent_spliced_in(list(inclusion_group1), list(inclusion_group2), list(exclusion_group), min_count=1)

test_psi_output<-readRDS(file="test_psi_output.rds")
all.equal(test_psi_output, check_psi_output)


######TS testing
inclusion_group1 <- SnaptronQueryBuilder$new()
inclusion_group1<-inclusion_group1$from_url("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr4:20763023-20763023&either=2&rfilter=strand:-")
inclusion_group2 <- SnaptronQueryBuilder$new()
inclusion_group2<-inclusion_group2$from_url("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr4:20763098-20763098&either=1&rfilter=strand:-")

check_ts_output<-tissue_specificity(list(inclusion_group1, inclusion_group2))
test_ts_output<-readRDS(file="test_ts_output.rds")
all.equal(test_ts_output, check_ts_output)

validate_sample_filter <- function(compilation, name, value) {
    if (is.null(registry <- pkg_globals$registry)) {
        return()
    }
    if (is.null(expected_type <- registry[[compilation]][[name]])) {
        field_names <- names(registry[[compilation]]) %>% sort()
        msg <- paste0(
            "Error: ", "`", name, "'",
            " is not a valid sample filter."
        )
        sorted_indexes <-
            utils::adist(field_names, name) %>% as.vector() %>% order()
        closest_match <- field_names[sorted_indexes][1]
        if (!identical(match, character(0))) {
            msg <-
                paste0(msg, " Perhaps you meant: ", closest_match, "?")
        }
    } else {
        type <- value_to_snaptron_type(value)
        if (expected_type == type ||
            (expected_type == "f" && type == "i")) {
            return(NULL)
        }
        else {
            msg <- paste0(
                "Error: ", "`", name, "'",
                " filter expects value of type ", make_verbose(expected_type),
                ", but got ", make_verbose(type)
            )
        }
    }

    msg
}

value_to_snaptron_type <- function(value) {
    n <- suppressWarnings(as.numeric(value))

    if (is.na(n)) {
        return("t")
    }

    if (is.wholenumber(n)) {
        return("i")
    }

    return("f")
}

make_verbose <- function(type) {
    switch(
        type,
        f = "Float",
        t = "String",
        i = "Integer"
    )
}

validate_range_filters <- function() {

}

SnaptronQueryBuilder <- R6Class("SnaptronQueryBuilder",
    public = list(
        initialize = function(...) {
            private$query <- list(...)
        },
        compilation = function(compilation = NULL) {
            if (!missing(compilation)) {
                assert_that(compilation %in% names(Compilation),
                            msg = paste0(compilation, ": is not a valid compilation"))
                private$query$compilation <- compilation
                invisible(self)
            } else {
                private$query$compilation
            }
        },
        regions = function(regions = NULL) {
            if (!missing(regions)) {
                if (is(regions, "GRanges")) {
                    private$query$regions <- regions
                } else if (is_genes_or_intervals(regions)) {
                    private$query$regions <- regions
                } else {
                    message <-
                        paste("regions must contain strings representing",
                              "HUGO genes or chromosome intervals, of the form",
                              "chr:start-end or chr:start-end:strand,",
                              "or GRanges object.")
                    private$query$regions <-
                        tryCatch(GenomicRanges::GRanges(regions),
                                 error = function(e) {
                                     stop(message)
                                 })
                }
                invisible(self)
            } else {
                private$query$regions
            }
        },
        row_filters = function(...) {
            if (!missing(...)) {
                private$query$row_filters <-
                    bool_expressions_to_strings(rlang::enquos(...))
                invisible(self)
            } else {
                private$query$row_filters
            }
        },
        column_filters = function(...) {
            if (!missing(...)) {
                private$query$column_filters <-
                    bool_expressions_to_strings(rlang::enquos(...))
                invisible(self)
            } else {
                private$query$column_filters
            }
        },
        sids = function(sids = NULL) {
            if (!missing(sids)) {
                assert_that(is.wholenumber(sids),
                    msg = "sids should be whole numbers")
                private$query$sids <- sids
                invisible(self)
            } else {
                private$query$sids
            }
        },
        coordinate_modifier = function(coordinate_modifier = NULL) {
            if (!missing(coordinate_modifier)) {
                private$query$coordinate_modifier <- coordinate_modifier
                invisible(self)
            } else {
                private$query$coordinate_modifier
            }
        },
        query_jx = function(return_rse = TRUE) {
            private$call("query_jx", list(sb = self, return_rse = return_rse))
        },
        query_exon = function(return_rse = TRUE) {
            private$call("query_exon", list(sb = self, return_rse = return_rse))
        },
        query_gene = function(return_rse = TRUE) {
            private$call("query_gene", list(sb = self, return_rse = return_rse))
        },
        from_url = function(url) {
            url <- httr::parse_url(url)
            if (url$hostname != "snaptron.cs.jhu.edu") {
                stop("URL does not point to Snaptron server", stop. = FALSE)
            }
            resp <- httr::HEAD(url)
            if (resp$status_code != 200 ||
                httr::http_type(resp) != "text/plain") {
                stop(sprintf("%s: is not a valid URL", url), call. = FALSE)
            }
            query <- list()
            for (i in seq_along(url$query)) {
                name <- switch(n <- names(url$query[i]),
                    rfilter = "row_filters",
                    sfilter = "column_filters",
                    regions = "regions",
                    n)

                if (name == "sids") {
                    query[[name]] <-
                        scan(textConnection(url$query[[i]]), sep = ",")
                } else if (name == "contains") {
                    if (url$query[[i]] == "1") {
                        query[["coordinate_modifier"]] <- Coordinates$Within
                    }
                } else if (name == "exact") {
                    if (url$query[[i]] == "1") {
                        query[["coordinate_modifier"]] <- Coordinates$Exact
                    }
                } else if (name == "either") {
                    if (url$query[[i]] == "1") {
                        query[["coordinate_modifier"]] <-
                            Coordinates$StartIsExactOrWithin
                    } else if (url$query[[i]] == "2") {
                        query[["coordinate_modifier"]] <-
                            Coordinates$EndIsExactOrWithin
                    }
                } else {
                    query[[name]] <- c(query[[name]], url$query[[i]])
                }
            }

            query$compilation <- strsplit(url$path, "/")[[1]][1]
            private$query <- query

            invisible(self)
        },
        print = function() {
            cat("<SnaptronQueryBuilder>\n")
            for (param in names(private$query)) {
                if (is.null(private$query[[param]])) {
                    next
                } else if (param == "coordinate_modifier") {
                    desc <- switch(private$query[[param]],
                                   Exact = "exact",
                                   Within = "contains",
                                   StartIsExactOrWithin = "either=1",
                                   EndIsExactOrWithin = "either=2",
                                   "overlaps")
                } else {
                    desc <- paste(private$query[[param]], collapse = ",")
                }
                cat("   ", param, ": ", desc, "\n", sep = "")
            }
        }
    ),
    private = list(
        query = list(),
        call = function(fn_name, args) {
            if (is.null(self$compilation())) {
                stop(
                    paste("Please set a compilation before running", fn_name),
                    call. = FALSE
                )
            }
            if (is.null(self$regions())) {
                stop(
                    paste("Please specify query regions before running", fn_name),
                    call. = FALSE
                )
            }
            fn <- get(fn_name, parent.frame())
            do.call(fn, args)
        }
    )
)

context("Test assertions")

test_that("Empty compilation", {
    sb <- SnaptronQueryBuilder$new()
    expect_error(sb$query_jx(), "Please set a compilation")
})

test_that("Empty region", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    expect_error(sb$query_jx(), "Please specify query regions")
})

test_that("Invalid chromosome range", {
    expect_error(query_jx(compilation = "tcga", regions = "ch0:100-200"))
    expect_error(query_jx(compilation = "tcga", regions = "chr23:100-200"))
    expect_error(query_jx(compilation = "tcga", regions = "chr22X:100-200"))
})

test_that("Invalid sample filters", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$column_filters(SNTS == "Brain")
    expect_error(query_gene(sb),
                 "(is not a valid sample filter)|(Invalid sample filter)")
})

test_that("Invalid compilation", {
    sb <- SnaptronQueryBuilder$new()
    expect_error(sb$compilation("abcde"), "abcde: is not a valid compilation")
})

test_that("Invalid coordinate modifier", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$coordinate_modifier("none")
    expect_error(query_exon(sb), "Invalid coordinate modifier")
})

test_that("Invalid SIDs", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    expect_error(sb$sids(c("1", "2")), "sids should be whole numbers")
    expect_error(sb$sids(c(1.2, 3.4)), "sids should be whole numbers")
})

context("Test query functions")

orig.options <- options(test_context = TRUE)

## test_that("regions and compilations are mandatory args", {
##     expect_error(query_jx(compilation = "tcga"), "argument \"regions\" is missing, with no default")
##     expect_error(query_jx(regions = "CD99"), "argument \"compilation\" is missing, with no default")
## })

test_that("simple junction query", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("srav2")
    sb$regions("CD99")

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(), "http://snaptron.cs.jhu.edu/srav2/snaptron?regions=CD99")
})

test_that("using genomic ranges", {
    x1 <- "chr2:100-200:-"
    g_range <- GenomicRanges::GRanges(x1)
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions(g_range)
    sb$column_filters(SMTS == "Brain")

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
                 "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=chr2:100-200&rfilter=strand:-&sfilter=SMTS:Brain")
})

test_that("junction query with NSE sample filter", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$column_filters(SMTS == "Brain")

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&sfilter=SMTS:Brain")
})

test_that("junction query with mutiple NSE sample filters", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("srav2")
    sb$regions("CD99")
    sb$column_filters(library_name == "HG00115.6", study_accession == "ERP001942")

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
        "http://snaptron.cs.jhu.edu/srav2/snaptron?regions=CD99&sfilter=library_name:HG00115.6&sfilter=study_accession:ERP001942")
})

test_that("junction query with one NSE range filter", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$row_filters(samples_count >= 5)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&rfilter=samples_count>:5")
})

test_that("junction query with mutiple NSE range filters", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$row_filters(samples_count <= 10, coverage_sum < 3)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&rfilter=samples_count<:10&rfilter=coverage_sum<3")
})

test_that("invalid sample filter name", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$column_filters(SNTS == "Brain")
    expect_error(
        query_jx(sb),
        "`SNTS' is not a valid sample filter")
})

test_that("invalid sample filter value", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$column_filters(SMTS == 2)
    expect_error(
        query_jx(sb),
        "`SMTS' filter expects value of type String, but got Integer")
})

test_that("junction query with sids", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$sids(1:3)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&sids=1,2,3")
})

test_that("query with non-numeric sids", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("tcga")
    sb$regions("CD99")
    expect_error(sb$sids(c("1", "2", "3")), msg = "sids should be whole numbers")
})

test_that("test coordinate Coordinates$Exact", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$coordinate_modifier(Coordinates$Exact)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
                 "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&exact=1")
})

test_that("test coordinate Coordinate$Within", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$coordinate_modifier(Coordinates$Within)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
                 "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&contains=1")
})

test_that("test coordinate Coordinates$StartIsExactorWithin", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$coordinate_modifier(Coordinates$StartIsExactOrWithin)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
                 "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&either=1")
})

test_that("test coordinate Coordinates$EndIsExactOrWithin", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    sb$coordinate_modifier(Coordinates$EndIsExactOrWithin)

    query_jx(sb)
    expect_equal(uri_of_last_successful_request(),
                 "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&either=2")
})

options(orig.options)

context("SnaptronQueryBuilder")

orig.options <- options()

test_that("empty query builder object", {
    sb <- SnaptronQueryBuilder$new()
    expect_output(sb$print(), "<SnaptronQueryBuilder>")
})

test_that("print method with set attributes", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("tcga")
    sb$regions("CD99")
    sb$sids(1:3)

    expect_output(sb$print(), "<SnaptronQueryBuilder>\\n   compilation: tcga\\n   regions: CD99\\n   sids: 1,2,3")
})

test_that("get/set methods", {
    sb <- SnaptronQueryBuilder$new()

    sb$compilation("gtex")
    expect_equal(sb$compilation(), "gtex")

    sb$regions("CD99")
    expect_equal(sb$regions(), "CD99")

    sb$row_filters(samples_count <= 10)
    expect_equal(as.character(sb$row_filters()), "samples_count <= 10")

    sb$column_filters(SMTS == "Brain")
    expect_equal(as.character(sb$column_filters()), "SMTS == Brain")

    sb$sids(1:10)
    expect_equal(sb$sids(), 1:10)
})

test_that("NSE rhs does not evaluate to basic type", {
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")
    sb$regions("CD99")
    expect_error(
        sb$column_filters(SMTS == c(1,2,3)),
        "does not evaluate to a basic type")
})

test_that("query methods", {
    options(test_context = TRUE)
    sb <- SnaptronQueryBuilder$new()
    sb$compilation("gtex")$regions("CD99")$column_filters(SMTS == "Brain")$row_filters(samples_count <= 10)$sids(c(50099,50102))

    sb$query_jx()
    expect_equal(uri_of_last_successful_request(), "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&rfilter=samples_count<:10&sfilter=SMTS:Brain&sids=50099,50102")

    sb$query_gene()
    expect_equal(uri_of_last_successful_request(), "http://snaptron.cs.jhu.edu/gtex/genes?regions=CD99&rfilter=samples_count<:10&sfilter=SMTS:Brain&sids=50099,50102")

    sb$query_exon()
    expect_equal(uri_of_last_successful_request(), "http://snaptron.cs.jhu.edu/gtex/exons?regions=CD99&rfilter=samples_count<:10&sfilter=SMTS:Brain&sids=50099,50102")
})

test_that("test building from invalid url", {
    sb <- SnaptronQueryBuilder$new()
    expect_error(sb$from_url("http://snap.cs.jhu.edu/gtex/snaptron?regions=CD99&rfilter=samples_count<:10"))
})

test_that("test building from url", {
    sb <- SnaptronQueryBuilder$new()
    sb$from_url("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99&rfilter=samples_count<:10&sfilter=SMTS:Brain&sids=50099,50102")

    expect_equal(sb$compilation(), "gtex")
    expect_equal(sb$regions(), "CD99")
    expect_equal(sb$row_filters(), "samples_count<:10")
    expect_equal(sb$column_filters(), "SMTS:Brain")
    expect_equal(sb$sids(), c(50099,50102))
})

test_that("test building from url with coordinate modifiers", {
    url <-
        "http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99"
    sb <- SnaptronQueryBuilder$new()

    sb$from_url(paste(url, "either=1", sep = "&"))
    expect_equal(sb$coordinate_modifier(), "StartIsExactOrWithin")
    expect_output(sb$print(), "<SnaptronQueryBuilder>\\n   regions: CD99\\n   coordinate_modifier: either=1\\n   compilation: gtex")

    sb$from_url(paste(url, "either=2", sep = "&"))
    expect_equal(sb$coordinate_modifier(), "EndIsExactOrWithin")
    expect_output(sb$print(), "<SnaptronQueryBuilder>\\n   regions: CD99\\n   coordinate_modifier: either=2\\n   compilation: gtex")

    sb$from_url(paste(url, "exact=1", sep = "&"))
    expect_equal(sb$coordinate_modifier(), "Exact")
    expect_output(sb$print(), "<SnaptronQueryBuilder>\\n   regions: CD99\\n   coordinate_modifier: exact\\n   compilation: gtex")

    sb$from_url(paste(url, "contains=1", sep = "&"))
    expect_equal(sb$coordinate_modifier(), "Within")
    expect_output(sb$print(), "<SnaptronQueryBuilder>\\n   regions: CD99\\n   coordinate_modifier: contains\\n   compilation: gtex")
})

## do this again in case there are any test failures and
## we are not able to restore the original env before
## failure
options(test_context = NULL)

#' Query Junctions/Genes/Exons
#'
#' Given one or more gene names or genomic range
#' intervals it will return a list of 0 or more genes, junctions, or exons
#' (depending on which query form is used) which overlap the ranges.
#'
#' @param sb A SnaptronQueryBuilder object
#' @param return_rse Should the query data be returned as a simple data frame or
#'   converted to a RangedSummarizedExperiment.
#'
#' @param split_by_region By default the results from multiple queries will be
#'   returned in a `RangedSummarizedExperiment` object with a `rowData` entry for
#'   each, labeling each result row according to the query it resulted
#'   from. However, if this is set to `TRUE`, the result will be a list of
#'   RangedSummarizedExperiment objects, one per original interval/gene. This
#'   latter option may be useful, but it requires a separate copy of the sample
#'   metadata for each original interval/gene.
#'
#' @examples
#' # Contruct a QueryBuilder object
#' qb <- QueryBuilder(compilation = "gtex", regions = "chr1:1-100000")
#' qb <- set_row_filters(qb, samples_count >= 20)
#' query_jx(qb)
#'
#' qb <- set_row_filters(qb, NULL)
#' qb <- set_column_filters(qb, SMTS == "Brain")
#' query_gene(qb)
#'
#' @return Functions will return either a RangedSummarizedExperiment or
#'   data.table depending on whether the \code{return_rse} parameter is set to
#'   \code{TRUE} or \code{FALSE}.
#' @export
query_jx <- function(sb, return_rse = TRUE, split_by_region = FALSE)
{
    strands <- NULL
    regions <- sb$regions()
    row_filters <- sb$row_filters()
    column_filters <- sb$column_filters()

    if (is(regions, "GRanges")) {
        strands <- extract_strands(sb$regions())
        regions <- extract_intervals(sb$regions())

        assert_that(length(regions) == length(strands))
    }

    should_bind <- length(regions) > 1 && !split_by_region
    res <- lapply(seq_along(regions), function(i) {
        if (!is.null(strands) && (strands[[i]] == "+" || strands[[i]] == "-")) {
            pos <- grep("strand", row_filters)
            if (!identical(pos, integer(0))) {
                row_filters[[pos]] <- paste0("strand:", strands[[i]])
            } else {
                row_filters <-
                    c(row_filters, paste0("strand:", strands[[i]]))
            }
        }

        run_query(compilation = sb$compilation(),
                  regions = regions[[i]],
                  row_filters = row_filters,
                  column_filters = column_filters,
                  coordinate_modifier = sb$coordinate_modifier(),
                  sids = sb$sids(),
                  return_rse = return_rse)
    })

    if (length(res) == 1) {
        res <- res[[1]]
    }
    if (should_bind) {
        rbind_func <- if (return_rse) SummarizedExperiment::rbind else rbind
        res <- do.call(rbind_func, res)
    }

    res
}

#' @rdname query_jx
#' @export
query_gene <- function(sb, return_rse = TRUE, split_by_region = FALSE)
{
    strands <- NULL
    regions <- sb$regions()
    row_filters <- sb$row_filters()
    column_filters <- sb$column_filters()

    if (is(regions, "GRanges")) {
        strands <- extract_strands(sb$regions())
        regions <- extract_intervals(sb$regions())

        assert_that(length(regions) == length(strands))
    }

    should_bind <- length(regions) > 1 && !split_by_region
    res <- lapply(seq_along(regions), function(i) {
        if (!is.null(strands) && (strands[i] == "+" || strands[[i]] == "-")) {
            pos <- grep("strand", row_filters)
            if (!identical(pos, integer(0))) {
                row_filters[pos] <- paste0("strand:", strands[[i]])
            } else {
                row_filters <- c(row_filters, paste0("strand:", strands[[i]]))
            }
        }

        run_query(compilation = sb$compilation(),
                  regions = regions[[i]],
                  row_filters = row_filters,
                  column_filters = column_filters,
                  coordinate_modifier = sb$coordinate_modifier(),
                  sids = sb$sids(),
                  endpoint = "genes",
                  return_rse = return_rse)
    })
    if (length(res) == 1) {
        res <- res[[1]]
    }
    if (should_bind) {
        rbind_func <- if (return_rse) SummarizedExperiment::rbind else rbind
        res <- do.call(rbind_func, res)
    }

    res
}

#' @rdname query_jx
#' @export
query_exon <- function(sb, return_rse = TRUE, split_by_region = FALSE)
{
    strands <- NULL
    regions <- sb$regions()
    row_filters <- sb$row_filters()
    column_filters <- sb$column_filters()

    if (is(regions, "GRanges")) {
        strands <- extract_strands(sb$regions())
        regions <- extract_intervals(sb$regions())


        assert_that(length(regions) == length(strands))
    }

    should_bind <- length(regions) > 1 && !split_by_region
    res <- lapply(seq_along(regions), function(i) {
        if (!is.null(strands) && (strands[i] == "+" || strands[i] == "-")) {
            pos <- grep("strand", row_filters)
            if (!identical(pos, integer(0))) {
                row_filters[pos] <- paste0("strand:", strands[i])
            } else {
                row_filters <- c(row_filters, paste0("strand:", strands[i]))
            }
        }

        run_query(compilation = sb$compilation(),
                  regions = regions[i],
                  row_filters = row_filters,
                  column_filters = column_filters,
                  coordinate_modifier = sb$coordinate_modifier(),
                  sids = sb$sids(),
                  endpoint = "exons",
                  return_rse = return_rse)
    })
    if (length(res) == 1) {
        res <- res[[1]]
    }
    if (should_bind) {
        rbind_func <- if (return_rse) SummarizedExperiment::rbind else rbind
        res <- do.call(rbind_func, res)
    }

    res
}

#' Return the URI of the last successful request to Snaptron
#'
#' @description This function can be paired with the \code{from_url} method from
#'   the QueryBuilder class, allowing users to share sources of data
#'   from Snaptron.
#' @return URI of last successful request to Snaptron or \code{NULL} if there
#'   have not been any successful requests.
#'
#' @export
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' query_jx(qb)
#' uri_of_last_successful_request()
uri_of_last_successful_request <- function() {
    pkg_globals$last_uri_accessed
}

get_compilation_metadata <- function(compilation) {
    assert_that(compilation %in% names(Compilation),
                msg = "Invalid compilation")

    if (is.null(pkg_globals$metadata[[compilation]])) {
        uri <- sprintf(
            "%s/%s/samples?all=1", pkg_globals$snaptron_host, compilation
        )
        tsv <- submit_query(uri)
        pkg_globals$metadata[[compilation]] <-
            data.table::fread(tsv, sep = "\t", quote = "")
    }

    pkg_globals$metadata[[compilation]]
}

is_logical_op <- function(op) {
    logical_ops <-
        c(
            as.symbol("=="),
            as.symbol("<="),
            as.symbol("<"),
            as.symbol(">"),
            as.symbol(">=")
        )
    any(vapply(logical_ops, identical, FUN.VALUE = logical(1), op))
}

tidy_filters <- function(filters) {
    filters <- gsub("==", ":", filters)
    filters <- gsub("=", ":", filters)
    filters <- gsub("\\s+", "", filters)

    filters
}

run_query <- function(compilation, regions, endpoint = "snaptron",
                      row_filters = NULL, column_filters = NULL, sids = NULL,
                      coordinate_modifier = NULL, construct_rse = TRUE,
                      return_rse = TRUE) {
    uri <- generate_snaptron_uri(
            compilation = compilation,
            regions = regions,
            endpoint = endpoint,
            row_filters = row_filters,
            column_filters = column_filters,
            coordinate_modifier = coordinate_modifier,
            sids = sids
    )

    if (!is.null(tc <- getOption("test_context")) && tc == TRUE) {
        assign("last_uri_accessed", uri, pkg_globals)
        return(NULL)
    } else {
        tsv <- submit_query(uri)
        assign("last_uri_accessed", uri, pkg_globals)
    }

    query_data <- data.table::fread(tsv, sep = "\t", header = TRUE)

    if (nrow(query_data) == 0) {
        warning(sprintf("query with uri: %s, returned no data.", uri))
        return(NULL)
    }

    if (!return_rse) {
        return(query_data)
    }

    metadata <- get_compilation_metadata(compilation)

    if (construct_rse == FALSE) {
        return(list(query_data = query_data, metadata = metadata))
    }

    rse(query_data, metadata)
}

generate_snaptron_uri <- function(compilation, regions,
                                  endpoint = "snaptron", row_filters = NULL,
                                  column_filters = NULL,
                                  coordinate_modifier = NULL, sids = NULL) {
    path <- paste(compilation, paste0(endpoint, "?"), sep = "/")
    query <- paste("regions", regions, sep = "=")

    if (!is.null(row_filters)) {
        query <- c(query, paste("rfilter",
                                tidy_filters(row_filters), sep = "="))
    }

    if (!is.null(column_filters)) {
        column_filters <- tidy_filters(column_filters)
        errors <- lapply(column_filters, function(filter) {
            fields <- stringr::str_split(filter, "\\W", n = 2)[[1]]
            validate_sample_filter(
                compilation,
                name = fields[[1]],
                value = fields[[2]]
            )
        }) %>% purrr::compact()

        if (length(errors) > 0) {
            error_string <- paste(errors, collapse = "\n")
            stop(error_string, call. = FALSE)
        }
        query <- c(query, paste("sfilter",
                                tidy_filters(column_filters),
                                sep = "="))
    }

    if (!is.null(coordinate_modifier)) {
        if (coordinate_modifier == Coordinates$Exact) {
            query <- c(query, paste("exact", "1", sep = "="))
        } else if (coordinate_modifier == Coordinates$Within) {
            query <- c(query, paste("contains", "1", sep = "="))
        } else if (coordinate_modifier == Coordinates$StartIsExactOrWithin) {
            query <- c(query, paste("either", "1", sep = "="))
        } else if (coordinate_modifier == Coordinates$EndIsExactOrWithin) {
            query <- c(query, paste("either", "2", sep = "="))
        } else {
            stop("Invalid coordinate modifier", stop. = FALSE)
        }
    }

    if (!is.null(sids)) {
        query <- c(query, paste("sids", paste(sids, collapse = ","), sep = "="))
    }

    paste0(pkg_globals$snaptron_host, path, paste(query, collapse = "&"))
}

extract_intervals <- function(g) {
    chr <- GenomicRanges::seqnames(g)
    beg <- GenomicRanges::start(g)
    end <- GenomicRanges::end(g)
    chr <- as.vector(rep(chr@values, chr@lengths))

    paste0(chr, ":", beg, "-", end)
}

extract_strands <- function(g) {
    strands <- GenomicRanges::strand(g)
    strands <- as.vector(rep(strands@values, strands@lengths))

    strands
}

submit_query <- function(uri) {
    resp <- httr::GET(uri)
    if (resp$status_code != "200"
        || resp$header[["content-type"]] != "text/plain") {
        stop("API did not return tsv", call. = FALSE)
    }
    rawToChar(resp$content)
}

convert_to_sparse_matrix <-
    function(rail_ids, unique_rail_ids, counts, samples_count, snaptron_ids) {
    i <- rep(seq_along(samples_count), samples_count)
    j <- match(rail_ids, unique_rail_ids)
    x <- counts

    dims <- c(length(snaptron_ids), length(unique_rail_ids))
    Matrix::sparseMatrix(i = i, j = j, x = x, dims = dims,
                         dimnames = list(snaptron_ids,
                                         paste0("rail_", unique_rail_ids)))
}

get_counts <- function(query_data) {
    rail_ids_and_counts <- extract_samples(query_data) %>%
        strsplit(":", fixed = TRUE)
    rail_ids <- as.numeric(vapply(rail_ids_and_counts, `[`, 1, FUN.VALUE = ""))
    unique_rail_ids <- sort(rail_ids) %>% unique()
    counts <- as.numeric(vapply(rail_ids_and_counts, `[`, 2, FUN.VALUE = ""))

    sparse_matrix <-
        convert_to_sparse_matrix(
            rail_ids, unique_rail_ids, counts,
            query_data$samples_count, query_data$snaptron_id
        )

    list(sparse_matrix, unique_rail_ids)
}

get_col_data <- function(metadata, rail_ids = NULL) {
    if (!is.null(rail_ids)) {
        metadata[metadata$rail_id %in% rail_ids]
    } else {
        metadata
    }
}

get_row_ranges <- function(query_data) {
    cols <- c("chromosome", "start", "end", "length", "strand", "samples")
    mcols <- query_data[, !cols, with = FALSE]

    GRanges(
        seqnames = query_data$chromosome,
        IRanges::IRanges(query_data$start, query_data$end),
        strand = query_data$strand,
        mcols
    )
}

rse <- function(query_data, metadata) {
    row_ranges <- get_row_ranges(query_data)
    count_data <- get_counts(query_data)
    col_data <- get_col_data(metadata, rail_ids = count_data[[2]])

    SummarizedExperiment(
        assays = list(counts = count_data[[1]]),
        rowRanges = row_ranges,
        colData = col_data,
        metadata = list(uri = uri_of_last_successful_request())
    )
}

extract_samples <- function(query_data) {
    unlist(lapply(strsplit(query_data$samples, ","), `[`, -1))
}

is.wholenumber <- function(x, tol = .Machine$double.eps^0.5) {
    assert_that(is.numeric(x))
    all(abs(x - round(x)) < tol)
}

## renaming second arg from values to value to
## appease R checks
`%<-%` <- function(bindings, value) {
    values <- force(value)
    bindings <- substitute(bindings)

    assert_that(length(bindings) == length(values) + 1)

    for (i in seq_along(values)) {
        var_name <- deparse(bindings[[i + 1]])
        assign(var_name, values[[i]], pos = parent.frame())
    }
}

bool_expressions_to_strings <- function(quosures) {
    if (length(quosures) == 1) {
        expr <- rlang::get_expr(quosures[[1]])
        if (is.null(expr) || rlang::is_bare_character(expr)) {
            return(expr)
        }
    }
    lapply(quosures, expression_to_string_helper)
}

expression_to_string_helper <- function(quosure) {
    env <- rlang::get_env(quosure)
    expr <- rlang::get_expr(quosure)

    if (rlang::is_bare_character(expr)) {
        return(expr)
    }

    assert_that(length(expr) == 3,
                msg = paste(deparse(expr), ": is not a valid filter"))
    assert_that(is_logical_op(expr[[1]]),
                msg = paste(deparse(expr), ": is not a bool expression"))
    assert_that(rlang::is_symbol(expr[[2]]),
                msg = paste(deparse(expr[[2]]), ": is not a valid name"))

    if (rlang::is_call(expr[[3]]) || rlang::is_symbol(expr[[3]])) {
        res <- rlang::eval_tidy(expr[[3]])
        assert_that(
            rlang::is_syntactic_literal(res),
            msg = paste(
                deparse(expr[[3]]),
                ": does not evaluate to a basic type"
            )
        )
        expr[[3]] <- res
    } else if (rlang::is_bare_character(expr[[3]])) {
        ## convert a string a symbol to prevent deparse from
        ## escaping quotes
        expr[[3]] <- rlang::sym(expr[[3]])
    }

    deparse(expr, backtick = FALSE)
}

## ----vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE----------------
library(snapcount)
## Track time spent on making the vignette
startTime <- Sys.time()

## ----'install', eval = FALSE--------------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("snapcount")
# 
# ## Check that you have a valid Bioconductor installation
# BiocManager::valid()

## ----'gene', eval = TRUE------------------------------------------------------

##Query coverage for gene, exon, and annotated junctions across all
#in the region of the CD99 gene
#from GTEx v6 sample compilation
#CD99 is chosen for its size
sb <- QueryBuilder(compilation="gtex", regions="CD99")
cd99.gene <- query_gene(sb)
dim(cd99.gene)
head(cd99.gene)

## ----'jx', eval = TRUE--------------------------------------------------------

##Query all exon-exon splice junctions within the region of gene CD99
cd99.jx.all <- query_jx(sb)
dim(cd99.jx.all)
cd99.jx.all

## ----'jx2', eval = TRUE-------------------------------------------------------

#now subfilter by sample tissue
#GTEx samples that are labeled with tissue type "Brain"
sb <- set_column_filters(sb, SMTS == "Brain")
cd99.jx.all <- query_jx(sb)
dim(cd99.jx.all)
head(cd99.jx.all)

## ----'exon', eval = TRUE------------------------------------------------------

cd99.exon <- query_exon(sb)
dim(cd99.exon)
head(cd99.exon)

## ----'jx3', eval = TRUE-------------------------------------------------------

###Only query junctions which are fully annotated---both left and
#right splice sites are found together in one or more of the
#Snaptron sourced annotations
sb <- set_row_filters(sb, annotated == 1)
cd99.jx <- query_jx(sb)
dim(cd99.jx)
head(cd99.jx)

## ----'md', eval = TRUE--------------------------------------------------------

##Metadata is stored directly in the RSE object.
#For example the library insert size can be retrieved
#across all runs  in the RSE
head(cd99.jx.all$InsertSize)

## ----'psi', eval = TRUE-------------------------------------------------------
#Build new query against GTEx
#left inclusion query
lq <- QueryBuilder(compilation="gtex", regions="chr19:45297955-45298142")
lq <- set_row_filters(lq, strand == "+")
lq <- set_coordinate_modifier(lq, Coordinates$Exact)
#right inclusion query
rq <- QueryBuilder(compilation="gtex", regions="chr19:45298223-45299810")
rq <- set_row_filters(rq, strand == "+")
rq <- set_coordinate_modifier(rq, Coordinates$Exact)
#exclusion query
ex <- QueryBuilder(compilation="gtex", regions="chr19:45297955-45299810")
ex <- set_row_filters(ex, strand == "+")
ex <- set_coordinate_modifier(ex, Coordinates$Exact)

psi <- percent_spliced_in(list(lq), list(rq), list(ex))
#order by psi descending
psi <- psi[order(-psi),]
head(psi)

## ----'jir', eval = TRUE-------------------------------------------------------
#groupA
A <- QueryBuilder(compilation="srav2", regions="chr2:29446395-30142858")
A <- set_row_filters(A, strand == "-")
A <- set_coordinate_modifier(A, Coordinates$Within)

#groupB
B <- QueryBuilder(compilation="srav2", regions="chr2:29416789-29446394")
B <- set_row_filters(B, strand == "-")
B <- set_coordinate_modifier(B, Coordinates$Within)

jir <- junction_inclusion_ratio(list(A),
                              list(B),
                              group_names=c("groupA","groupB"))
head(jir)

## ----'ssc', eval = TRUE-------------------------------------------------------
## We define the left/right splice junction supports of 3
#cassette exons for the following 3 genes:

## "left"/"right" is relative to the forward strand,
#reference genome coordinates,
#we use this terminology instead of 5' or 3'
#since the gene may be on the reverse strand.

###GNB1
GNB1l <- QueryBuilder(compilation="gtex", regions="chr1:1879786-1879786")
GNB1l <- set_row_filters(GNB1l, strand == "-")
GNB1l <- set_coordinate_modifier(GNB1l, Coordinates$EndIsExactOrWithin)

GNB1r <- QueryBuilder(compilation="gtex", regions="chr1:1879903-1879903")
GNB1r <- set_row_filters(GNB1r, strand == "-")
GNB1r <- set_coordinate_modifier(GNB1r, Coordinates$StartIsExactOrWithin)

###PIK3CD
PIK3l <- QueryBuilder(compilation="gtex", regions="chr1:9664595-9664595")
PIK3l <- set_row_filters(PIK3l, strand == "+")
PIK3l <- set_coordinate_modifier(PIK3l, Coordinates$EndIsExactOrWithin)

PIK3r <- QueryBuilder(compilation="gtex", regions="chr1:9664759-9664759")
PIK3r <- set_row_filters(PIK3r, strand == "+")
PIK3r <- set_coordinate_modifier(PIK3r, Coordinates$StartIsExactOrWithin)

###TAP2
T2l <- QueryBuilder(compilation="gtex", regions="chr6:32831148-32831148")
T2l <- set_row_filters(T2l, strand == "-")
T2l <- set_coordinate_modifier(T2l, Coordinates$EndIsExactOrWithin)

T2r <- QueryBuilder(compilation="gtex", regions="chr6:32831182-32831182")
T2r <- set_row_filters(T2r, strand == "-")
T2r <- set_coordinate_modifier(T2r, Coordinates$StartIsExactOrWithin)

ssc_func <- shared_sample_counts
ssc <- ssc_func(list(GNB1l, GNB1r),
              list(PIK3l, PIK3r),
              list(T2l, T2r),
              group_names=c("validated","not validated","validated"))
ssc


## ----'ts1', eval = TRUE-------------------------------------------------------
A <- QueryBuilder(compilation="gtex", regions="chr4:20763023-20763023")
A <- set_row_filters(A, strand == "-")
A <- set_coordinate_modifier(A, Coordinates$EndIsExactOrWithin)

ts <- tissue_specificity(list(A))
head(ts)

## ----'ts2', eval = TRUE-------------------------------------------------------
A <- QueryBuilder(compilation="gtex", regions="chr4:20763023-20763023")
A <- set_row_filters(A, strand == "-")
A <- set_coordinate_modifier(A, Coordinates$EndIsExactOrWithin)

B <- QueryBuilder(compilation="gtex", regions="chr4:20763098-20763098")
B <- set_row_filters(B, strand == "-")
B <- set_coordinate_modifier(B, Coordinates$StartIsExactOrWithin)

ts <- tissue_specificity(list(A, B))
head(ts)

## ----'jm', eval = TRUE--------------------------------------------------------
gtex <- QueryBuilder(compilation="gtex", regions="chr1:1879786-1879786")
gtex <- set_row_filters(gtex, strand == "-")
gtex <- set_coordinate_modifier(gtex, Coordinates$EndIsExactOrWithin)

tcga <- QueryBuilder(compilation="tcga", regions="chr1:1879786-1879786")
tcga <- set_row_filters(tcga, strand == "-")
tcga <- set_coordinate_modifier(tcga, Coordinates$EndIsExactOrWithin)

gtex_tcga_union <- junction_union(gtex,tcga)
dim(gtex_tcga_union)
head(gtex_tcga_union)

## ----'ji', eval = TRUE--------------------------------------------------------
gtex <- QueryBuilder(compilation="gtex", regions="chr1:1879786-1879786")
gtex <- set_row_filters(gtex, strand == "-")
gtex <- set_coordinate_modifier(gtex, Coordinates$EndIsExactOrWithin)

tcga <- QueryBuilder(compilation="tcga", regions="chr1:1879786-1879786")
tcga <- set_row_filters(tcga, strand == "-")
tcga <- set_coordinate_modifier(tcga, Coordinates$EndIsExactOrWithin)

gtex_tcga_intersection <- junction_intersection(gtex,tcga)
dim(gtex_tcga_intersection)
head(gtex_tcga_intersection)

# nocov start
#' Construct a QueryBuilder object given a compilation and one or regions.
#'
#' @param compilation A single string containing the name of the
#'   Snaptron data source. Any variant of the `Compilation` enum can also be
#'   passed an argument.
#' @param regions Either a list of 1 more `HUGO` gene names as strings
#'   `e.g. "BRCA1"` or a Granges class object containing one or more geneomic
#'   intervals `(e.g. "chr1:1-1000")`.
#'
#' @return
#' A QueryBuilder object.
#' @examples
#' # contruct a query builder for GTEX data source and BRAC1 gene
#' qb <- QueryBuilder(compilation = Compilation$gtex, regions = "BRCA1")
#'
#' # contruct a query builder for TCGA data source and chromosome region
#' qb <- QueryBuilder(compilation = "tcga", regions = "chr1:1-1000")
#'
#' # construct a query builder for TCGA data source using GRanges object
#' library(GenomicRanges)
#' qb <- QueryBuilder(compilation = "tcga", regions = GRanges("chr1", "1-1000"))
#' @export
QueryBuilder <- function(compilation, regions) {
    SnaptronQueryBuilder$new(compilation = compilation, regions = regions)
}

#' Get or set query compilation
#'
#' @param qb A QueryBuilder object constructed using the
#'   \code{\link{QueryBuilder}} function.
#' @param compilation A single string containing the name of the Snaptron data
#'   source. Any variant of the `Compilation` enum can also be passed as an
#'   argument.
#'
#' @return
#' \code{get_compilation} returns the current compilation as string.
#' \code{set_compilation} returns a new \code{QueryBuilder} object with
#'   the compilation set to the value of \code{compilation}.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' get_compilation(qb)
#' qb <- set_compilation(qb, Compilation$tcga)
#' get_compilation(qb)
#' @export
get_compilation <- function(qb) {
    qb$compilation()
}

#' Get or set query regions
#'
#' @param qb A QueryBuilder object constructed using the
#'   \code{\link{QueryBuilder}} function.
#' @param regions Either a list of 1 more `HUGO` gene names as strings
#'   `e.g. "BRCA1"` or a Granges class object containing one or more geneomic
#'   intervals `(e.g. "chr1:1-1000")`.
#'
#' @return
#' \code{get_regions} returns the current regions as a list of strings.
#' \code{set_regions} returns a new \code{QueryBuilder} object with the
#'   regions set to the value of \code{regions}.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' get_regions(qb)
#' qb <- set_regions(qb, "chr1:1-1000")
#' get_regions(qb)
#' qb <- set_regions(qb, GenomicRanges::GRanges("chr1", "1-1000"))
#' get_regions(qb)
#' @export
get_regions <- function(qb) {
    qb$regions()
}

#' Get or set sample-related contraints for query
#'
#' @param qb a QueryBuilder object constructed using the
#'   \code{\link{QueryBuilder}} function
#' @param ... one or more boolean predicates as either strings or unevaluated
#'   expressions
#'
#' @return
#' \code{get_column_filters} returns the current filters as a list of strings.
#' \code{set_column_filters} returns a new \code{QueryBuilder} object
#'   with the column filters set to the value of \code{column_filters}.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' # column filters set using a string
#' qb <- set_column_filters(qb, "SMTS == Brain")
#' get_column_filters(qb)
#' # column filters set using unevaluated expression
#' qb <- set_column_filters(qb, SMTS == "Spleen")
#' get_column_filters(qb)
#' @export
get_column_filters <- function(qb) {
    qb$column_filters()
}

#' Get or set range-related contraints for query
#'
#' @param qb a QueryBuilder object constructed using the
#'   \code{\link{QueryBuilder}} function.
#' @param ... one or more boolean predicates as either strings or unevaluated
#'   expressions.
#'
#' @return
#' \code{get_row_filters} returns the current row filters as list of strings.
#' \code{set_row_filters} returns a new \code{QueryBuilder} object with
#'   the row filters set to the value of \code{row_filters}.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' # row filters set as a string
#' qb <- set_row_filters(qb, "strand == +")
#' get_row_filters(qb)
#' # row filters set using unevaluated expression
#' qb <- set_row_filters(qb, strand == "+")
#' get_row_filters(qb)
#' @export
get_row_filters <- function(qb) {
    qb$row_filters()
}

#' Get or set query sample ids
#'
#' @param qb a QueryBuilder object constructed using the
#'   \code{\link{QueryBuilder}} function.
#' @param sids a vector or 1 or more whole numbers to filter results on.
#'
#' @return
#' \code{get_sids} returns the current sample ids as a vector of integers.
#' \code{set_sids} returns a new \code{QueryBuilder} object with the
#'   sample ids set to the value of \code{sids}.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' qb <- set_sids(qb, c(1, 2, 3))
#' get_sids(qb)
#' @export
get_sids <- function(qb) {
    qb$sids()
}

#' Get or set coordinate modifiers for the query.
#'
#' @param qb a QueryBuilder object constructed using the
#'   \code{\link{QueryBuilder}} function.
#' @param coordinate_modifier any of the variants of the \link{Coordinates}
#'   enum.
#'
#' @return
#' \code{get_coordinate_modifier} returns the current coodinate modifier
#'    as a string.
#' \code{set_coordinate_modifier} returns a new \code{QueryBuilder}
#'   object with the coordinate modifier set to the value of
#'   \code{coordinate_modifier}.
#'
#' @examples
#' qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
#' qb <- set_coordinate_modifier(qb, Coordinates$Within)
#' get_coordinate_modifier(qb)
#' @export
get_coordinate_modifier <- function(qb) {
    qb$coordinate_modifier()
}

#' @rdname get_compilation
#' @export
set_compilation <- function(qb, compilation) {
    cloned_qb <- qb$clone(deep = TRUE)$compilation(compilation)
    cloned_qb
}

#' @rdname get_regions
#' @export
set_regions <- function(qb, regions) {
    cloned_qb <- qb$clone(deep = TRUE)$regions(regions)
    cloned_qb
}

#' @rdname get_column_filters
#' @export
set_column_filters <- function(qb, ...) {
    cloned_qb <- qb$clone(deep = TRUE)$column_filters(...)
    cloned_qb
}

#' @rdname get_row_filters
#' @export
set_row_filters <- function(qb, ...) {
    cloned_qb <- qb$clone(deep = TRUE)$row_filters(...)
    cloned_qb
}

#' @rdname get_sids
#' @export
set_sids <- function(qb, sids) {
    cloned_qb <- qb$clone(deep = TRUE)$sids(sids)
    cloned_qb
}

#' @rdname get_coordinate_modifier
#' @export
set_coordinate_modifier <- function(qb, coordinate_modifier) {
    cloned_qb <- qb$clone(deep = TRUE)$coordinate_modifier(coordinate_modifier)
    cloned_qb
}

#' Constructs a \code{QueryBuilder} object from the given url
#'
#' @param url a well-formed url preferably obtained from a call to the
#'   \code{\link{uri_of_last_successful_request}} function
#'
#' @return
#' Returns a \code{QueryBuilder} object with attributes set from the
#'   parsed url.
#'
#' @examples
#' sb <- from_url("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99")
#' get_regions(sb)
#' get_compilation(sb)
#' @export
from_url <- function(url) {
    SnaptronQueryBuilder$new()$from_url(url)
}

#nocov end

context("QueryBuilderWrappers")

global_qb <- QueryBuilder(compilation = "gtex", regions = "CD99")

test_that("test initializer", {
    qb <- QueryBuilder(compilation = "gtex", regions = "CD99")

    expect_equal(qb$compilation(), "gtex")
    expect_equal(qb$regions(), "CD99")
})

test_that("test get/set compilation", {
    qb <- set_compilation(global_qb, "tcga")

    expect_equal(get_compilation(qb), "tcga")
})

## test_that("test get/set regions", {
##     qb <- set_regions(global_qb, "chr4:1-1000")

##     expect_equal(get_regions(qb), "ch4:1-1000")
## })

test_that("test get/set column filters as strings", {
    qb <- set_column_filters(global_qb, "foo == bar", "a < b")

    filters <- get_column_filters(qb)
    expect_length(filters, 2)
    expect_equal(filters[[1]], "foo == bar")
    expect_equal(filters[[2]], "a < b")
})

test_that("test get/set column filters as expressions", {
    qb <- set_column_filters(global_qb, foo == "bar", a < (1 + 2))

    filters <- get_column_filters(qb)
    expect_length(filters, 2)
    expect_equal(filters[[1]], "foo == bar")
    expect_equal(filters[[2]], "a < 3")

})

test_that("test get/set row filters as strings", {
    qb <- set_row_filters(global_qb, "foo == bar", "a < b")

    filters <- get_row_filters(qb)
    expect_length(filters, 2)
    expect_equal(filters[[1]], "foo == bar")
    expect_equal(filters[[2]], "a < b")
})

test_that("test get/set row filters as expressions", {
    qb <- set_row_filters(global_qb, foo == "bar", a < (1 + 2))

    filters <- get_row_filters(qb)
    expect_length(filters, 2)
    expect_equal(filters[[1]], "foo == bar")
    expect_equal(filters[[2]], "a < 3")

})

test_that("get get/set sids", {
    qb <- set_sids(global_qb, 1:10)

    expect_equal(get_sids(qb), 1:10)
})

test_that("get/set coordinate_modifiers", {
    qb <- set_coordinate_modifier(global_qb, Coordinates$Exact)

    expect_equal(get_coordinate_modifier(qb), "Exact")
})

test_that("query builder from url", {
    qb <- from_url("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99")

    expect_equal(get_compilation(qb), "gtex")
    expect_equal(get_regions(qb), "CD99")
})

---
title: "snapcount vignette"
author:
  - name: Christopher Wilks
    affiliation:
    - &jhucs Johns Hopkins University Computer Science
    email: chris.wilks@jhu.edu
output:
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    code_folding: show
date: "`r doc_date()`"
package: "`r pkg_ver('snapcount')`"
vignette: >
  %\VignetteIndexEntry{snapcount quick start guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  R
---

```{r vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE}
library(snapcount)
## Track time spent on making the vignette
startTime <- Sys.time()
```

# Why would you want to use Snapcount?

snapcount is directed at current and potential `r Biocpkg('recount')` users who want the further ability to quickly filter RNA-seq derived coverage data to just the regions of the human genome and the samples they care about.

This avoids having to download a whole study's worth of data across the entire genome.

While it offers the same datasets that `r Biocpkg('recount')` contains, it enables querying via a particular gene or genomic coordinate range entered by the user and only downloading the results from that query.

It then builds a RangedSummarizedExperiment (RSE) object on the fly, based on just the rows/columns in those results.

This query functionality is supported for genes, exons, and exon-exon junctions.

In addition, a set of aggregation functions are available for exon-exon junction data to allow for further filtering and summarizing.

# Basics

## Install snapcount

`R` is an open-source statistical environment which can be easily modified to enhance its functionality via packages.
`R` can be installed on any operating system from [CRAN](https://cran.r-project.org/) after which you can install snapcount
by using the following commands in your `R` session:


```{r 'install', eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("snapcount")

## Check that you have a valid Bioconductor installation
BiocManager::valid()
```


## Required knowledge

snapcount is based on other packages and in particular in those that have implemented the infrastructure needed for dealing with RNA-seq data. That is, packages like `r Biocpkg('GenomicFeatures')` and `r Biocpkg('recount')` that allow you to import the data. A snapcount user is not expected to deal with those packages directly but will need to be familiar with `r Biocpkg('SummarizedExperiment')` to understand the results snapcount generates.  This vignette is based partially on the [recount2 quick start guide](https://bioconductor.org/packages/recount).

If you are asking yourself the question "Where do I start using Bioconductor?" you might be interested in [this blog post](http://lcolladotor.github.io/2014/10/16/startbioc/).

## Asking for help

As package developers, we try to explain clearly how to use our packages and in which order to use the functions. But `R` and `Bioconductor` have a steep learning curve so it is critical to learn where to ask for help. The blog post quoted above mentions some but we would like to highlight the [Bioconductor support site](https://support.bioconductor.org/) as the main resource for getting help: remember to use the `snapcount` tag and check [the older posts](https://support.bioconductor.org/t/snapcount/). Other alternatives are available such as creating GitHub issues and tweeting. However, please note that if you want to receive help you should adhere to the [posting guidelines](http://www.bioconductor.org/help/support/posting-guide/). It is particularly critical that you provide a small reproducible example and your session information so package developers can track down the source of the error.

snapcount is an interface to the Snaptron webservices and data.
For questions regarding Snaptron itself and/or its data, please use the [Gitter channel](https://gitter.im/snaptron/Lobby).

## Citing snapcount

We hope that snapcount will be useful for your research. Please use the following information to cite the package specifically, thank you!

```
Christopher Wilks, Rone Charles, Ben Langmead (2019).
snapcount: R/Bioconductor Package for interfacing with Snaptron
for rapid quering of expression counts.
https://github.com/langmead-lab/snapcount
```

The Snaptron project itself is published separately and can be cited with the following:
```
Wilks, C, Gaddipati, P, Nellore, A, Langmead, B (2018).
Snaptron: querying splicing patterns across tens
of thousands of RNA-seq samples.
Bioinformatics, 34, 1:114-116.
```

# Introduction

snapcount makes it easy to query the [Snaptron web services](http://snaptron.cs.jhu.edu/), with results presented as `r Biocpkg('RangedSummarizedExperiment')` objects.
You can query measurements for **genes**, **exons**, **splice junctions** and **coverage vectors** from the RNA-seq samples indexed in Snaptron.
Samples are organized into compilations (e.g. `srav2`) that altogether contain the same studies and summaries in the `r Biocpkg('recount')` resource.  Queries can be filtered to narrow the focus to particular genes or genomic intervals, to events with certain prevalence, to events that do or don't appear in gene annotation, or to samples with particular metadata.

snapcount complements the `r Biocpkg('recount')` package, which also allows for searching by coordinates or by HUGO gene names.
In general, `r Biocpkg('recount')` works best when you are interested in *all* the genes, exons, or splice junctions in a study, whereas snapcount is best for queries over a particular subset of genes or intervals across all or most of the samples in recount2/Snaptron.  The more specific your query, the faster and easier it will be to use snapcount.

All the RNA-seq samples in the recount and Snaptron resources were analyzed using the [Rail-RNA](http://rail.bio) aligner.  Rail produces spliced alignments that in turn produce coverage and junction-level summaries that are further processed to obtain the data in recount and Snaptron.

Similar to the `r Biocpkg('recount')` all coverage counts in Snaptron/snapcount are stored/retrieved as raw, un-normalized counts.

# Basic queries

A basic query returns a *RangedSummarizedExperiment* (RSE) object with one or more features (genes, exons, or splice junctions) as *rowRanges*.
Raw coverage counts are returned as the *counts* assay in the RSE  Full sample metadata is returned as the  *colData* of the RSE

Basic queries include:

* exon-exon splice junction counts: *query_jx*
* exon-level quantifications: *query_exon*
* genes-level quantifications: *query_gene*

The Gencode v25 annotation defines what genes and exons can be queried (as in `r Biocpkg('recount')`).
For splice junctions, both annotated and novel junctions are queried at the same time unless one is explicitly filtered.

Metadata columns will vary by compilation (e.g. TCGA vs. GTEx).
Please be cautious with metadata.  We strive to make it as complete and usable as possible, but it can still be incomplete, incorrect or inconsistently formatted (e.g. "age" in the `srav2` compilation).

Start by querying for all junctions within the region of gene CD99:
```{r 'gene', eval = TRUE}

##Query coverage for gene, exon, and annotated junctions across all
#in the region of the CD99 gene
#from GTEx v6 sample compilation
#CD99 is chosen for its size
sb <- QueryBuilder(compilation="gtex", regions="CD99")
cd99.gene <- query_gene(sb)
dim(cd99.gene)
head(cd99.gene)
```

Now query for junctions:
```{r 'jx', eval = TRUE}

##Query all exon-exon splice junctions within the region of gene CD99
cd99.jx.all <- query_jx(sb)
dim(cd99.jx.all)
cd99.jx.all
```

Now query for junctions and filter by sample type: Brain:
```{r 'jx2', eval = TRUE}

#now subfilter by sample tissue
#GTEx samples that are labeled with tissue type "Brain"
sb <- set_column_filters(sb, SMTS == "Brain")
cd99.jx.all <- query_jx(sb)
dim(cd99.jx.all)
head(cd99.jx.all)
```

Same query/filter again but for exons:
```{r 'exon', eval = TRUE}

cd99.exon <- query_exon(sb)
dim(cd99.exon)
head(cd99.exon)
```

Now query junctions but further filter for only those which are fully annotated:
```{r 'jx3', eval = TRUE}

###Only query junctions which are fully annotated---both left and
#right splice sites are found together in one or more of the
#Snaptron sourced annotations
sb <- set_row_filters(sb, annotated == 1)
cd99.jx <- query_jx(sb)
dim(cd99.jx)
head(cd99.jx)
```

Now check an example of the metadata stored in the RSE, InsertSize:
```{r 'md', eval = TRUE}

##Metadata is stored directly in the RSE object.
#For example the library insert size can be retrieved
#across all runs  in the RSE
head(cd99.jx.all$InsertSize)
```

# High level queries

"High-level" queries answer more complex questions about splicing patterns.  Results of high level queries are data frames unless otherwise specified.

## Percent spliced in (PSI)

PSI measures how often an exon is included in the surrounding transcript.
This query considers only the common case
where a cassette exon has two inclusion junctions (junctions that join the exon to its up- and downstream neighbors) and one exclusion junction (that goes between the neighbors while skipping the exon).
Normal PSI values range from 0 (never included) to 1 (always included).
The value -1 is returned for a
sample where either one or the other inclusion groups had 0 coverage or the total raw coverage
across groups was less than a specified minimum count (`min_count` defaults to 20).

```{r 'psi', eval = TRUE}
#Build new query against GTEx
#left inclusion query
lq <- QueryBuilder(compilation="gtex", regions="chr19:45297955-45298142")
lq <- set_row_filters(lq, strand == "+")
lq <- set_coordinate_modifier(lq, Coordinates$Exact)
#right inclusion query
rq <- QueryBuilder(compilation="gtex", regions="chr19:45298223-45299810")
rq <- set_row_filters(rq, strand == "+")
rq <- set_coordinate_modifier(rq, Coordinates$Exact)
#exclusion query
ex <- QueryBuilder(compilation="gtex", regions="chr19:45297955-45299810")
ex <- set_row_filters(ex, strand == "+")
ex <- set_coordinate_modifier(ex, Coordinates$Exact)

psi <- percent_spliced_in(list(lq), list(rq), list(ex))
#order by psi descending
psi <- psi[order(-psi),]
head(psi)
```


## Junction Inclusion Ratio (JIR)

Junction Inclusion Ratio (JIR) measures the relative prevalence of two splicing patterns.  It is similar to, but more general than the PSI query.
The query defines two sets of junctions to be compared.  The sets may or may not have some junctions in common.
JIR ranges from -1.0 to 1.0, where 0 says that the total number of reads supporting the junctions in set A equals the number of reads supporting set-B junctions.  JIR approaches 1 when there are many more reads supporting the set-B junctions compared to the set-A junctions.  JIR approaches -1 when set-A junctions have many more supporting reads than set-B junctions.
The list of results is presented in descending order by JIR.

```{r 'jir', eval = TRUE}
#groupA
A <- QueryBuilder(compilation="srav2", regions="chr2:29446395-30142858")
A <- set_row_filters(A, strand == "-")
A <- set_coordinate_modifier(A, Coordinates$Within)

#groupB
B <- QueryBuilder(compilation="srav2", regions="chr2:29416789-29446394")
B <- set_row_filters(B, strand == "-")
B <- set_coordinate_modifier(B, Coordinates$Within)

jir <- junction_inclusion_ratio(list(A),
                              list(B),
                              group_names=c("groupA","groupB"))
head(jir)
```

## Shared Sample Count (SSC)

SSC reports the number of samples where one or more sets of junctions co-occur.  We say two junctions co-occur in a sample of they both have at least one supporting read in that sample.  The junctions in the set might describe a particular alternative splicing event, but they do not have to.  The sets of junctions are defined using basic queries.

```{r 'ssc', eval = TRUE}
## We define the left/right splice junction supports of 3
#cassette exons for the following 3 genes:

## "left"/"right" is relative to the forward strand,
#reference genome coordinates,
#we use this terminology instead of 5' or 3'
#since the gene may be on the reverse strand.

###GNB1
GNB1l <- QueryBuilder(compilation="gtex", regions="chr1:1879786-1879786")
GNB1l <- set_row_filters(GNB1l, strand == "-")
GNB1l <- set_coordinate_modifier(GNB1l, Coordinates$EndIsExactOrWithin)

GNB1r <- QueryBuilder(compilation="gtex", regions="chr1:1879903-1879903")
GNB1r <- set_row_filters(GNB1r, strand == "-")
GNB1r <- set_coordinate_modifier(GNB1r, Coordinates$StartIsExactOrWithin)

###PIK3CD
PIK3l <- QueryBuilder(compilation="gtex", regions="chr1:9664595-9664595")
PIK3l <- set_row_filters(PIK3l, strand == "+")
PIK3l <- set_coordinate_modifier(PIK3l, Coordinates$EndIsExactOrWithin)

PIK3r <- QueryBuilder(compilation="gtex", regions="chr1:9664759-9664759")
PIK3r <- set_row_filters(PIK3r, strand == "+")
PIK3r <- set_coordinate_modifier(PIK3r, Coordinates$StartIsExactOrWithin)

###TAP2
T2l <- QueryBuilder(compilation="gtex", regions="chr6:32831148-32831148")
T2l <- set_row_filters(T2l, strand == "-")
T2l <- set_coordinate_modifier(T2l, Coordinates$EndIsExactOrWithin)

T2r <- QueryBuilder(compilation="gtex", regions="chr6:32831182-32831182")
T2r <- set_row_filters(T2r, strand == "-")
T2r <- set_coordinate_modifier(T2r, Coordinates$StartIsExactOrWithin)

ssc_func <- shared_sample_counts
ssc <- ssc_func(list(GNB1l, GNB1r),
              list(PIK3l, PIK3r),
              list(T2l, T2r),
              group_names=c("validated","not validated","validated"))
ssc

```


## Tissue Specificity (TS)

This query asks if a particular set of junctions seems to be associated with tissue type.
This query defaults to using the `gtex` compilation, but it could work with any compilation having a `tissue` metadata field (e.g. `tcga`).

The main output is a list of all the samples in the compilation with either a 0 or a 1 in the *shared* column.
1 indicates that a junction from the query occurs in that sample (tissue type).

This output can be passed to a statistical test such as an F-test to assess the significance of tissue enrichment.


```{r 'ts1', eval = TRUE}
A <- QueryBuilder(compilation="gtex", regions="chr4:20763023-20763023")
A <- set_row_filters(A, strand == "-")
A <- set_coordinate_modifier(A, Coordinates$EndIsExactOrWithin)

ts <- tissue_specificity(list(A))
head(ts)
```

The TS function also supports more than one set of junctions.

The main output is a list of all the samples in the compilation with either a 0 or a 1 in the *shared* column.
1 here indicates that at least one junction from all sets occurred in that sample, i.e. the sample is shared across the results of all the junction queries.

A use case of the 2-set TS function is to measure the significance of tissue enrichment across junctions which support a cassette exon.


```{r 'ts2', eval = TRUE}
A <- QueryBuilder(compilation="gtex", regions="chr4:20763023-20763023")
A <- set_row_filters(A, strand == "-")
A <- set_coordinate_modifier(A, Coordinates$EndIsExactOrWithin)

B <- QueryBuilder(compilation="gtex", regions="chr4:20763098-20763098")
B <- set_row_filters(B, strand == "-")
B <- set_coordinate_modifier(B, Coordinates$StartIsExactOrWithin)

ts <- tissue_specificity(list(A, B))
head(ts)
```

## Junction Union (Merge) and Intersection

The junction union query provides the ability to get a single RSE object of non-redundant junctions across multiple compilations which share a common reference (e.g. HG38).
Junctions which are the same (have the same chromosome coordinates and strand) are merged into a single row.

An example would be search for the same junctions across the human HG38-based compilations, GTEx and TCGA.
This can be used to efficiently leverage GTEx as a substitute "normal" control, while searching for splicing phenomena specific to the tumor samples in TCGA.

In the following example we query for all junctions that have a specific 5' (donor) coordinate in both compilations, merging the results.
Note, this query, due to the number of samples involved (approximately 18,000) takes several seconds to complete.

```{r 'jm', eval = TRUE}
gtex <- QueryBuilder(compilation="gtex", regions="chr1:1879786-1879786")
gtex <- set_row_filters(gtex, strand == "-")
gtex <- set_coordinate_modifier(gtex, Coordinates$EndIsExactOrWithin)

tcga <- QueryBuilder(compilation="tcga", regions="chr1:1879786-1879786")
tcga <- set_row_filters(tcga, strand == "-")
tcga <- set_coordinate_modifier(tcga, Coordinates$EndIsExactOrWithin)

gtex_tcga_union <- junction_union(gtex,tcga)
dim(gtex_tcga_union)
head(gtex_tcga_union)
```

The junction intersection query provides similar functionality but restricting the output RSE object to only those junctions found in both compilations.

Using the same scenario as above, but this time the result set will be reduced.

```{r 'ji', eval = TRUE}
gtex <- QueryBuilder(compilation="gtex", regions="chr1:1879786-1879786")
gtex <- set_row_filters(gtex, strand == "-")
gtex <- set_coordinate_modifier(gtex, Coordinates$EndIsExactOrWithin)

tcga <- QueryBuilder(compilation="tcga", regions="chr1:1879786-1879786")
tcga <- set_row_filters(tcga, strand == "-")
tcga <- set_coordinate_modifier(tcga, Coordinates$EndIsExactOrWithin)

gtex_tcga_intersection <- junction_intersection(gtex,tcga)
dim(gtex_tcga_intersection)
head(gtex_tcga_intersection)
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{get_compilation}
\alias{get_compilation}
\alias{set_compilation}
\title{Get or set query compilation}
\usage{
get_compilation(qb)

set_compilation(qb, compilation)
}
\arguments{
\item{qb}{A QueryBuilder object constructed using the
\code{\link{QueryBuilder}} function.}

\item{compilation}{A single string containing the name of the Snaptron data
source. Any variant of the \code{Compilation} enum can also be passed as an
argument.}
}
\value{
\code{get_compilation} returns the current compilation as string.
\code{set_compilation} returns a new \code{QueryBuilder} object with
the compilation set to the value of \code{compilation}.
}
\description{
Get or set query compilation
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
get_compilation(qb)
qb <- set_compilation(qb, Compilation$tcga)
get_compilation(qb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{QueryBuilder}
\alias{QueryBuilder}
\title{Construct a QueryBuilder object given a compilation and one or regions.}
\usage{
QueryBuilder(compilation, regions)
}
\arguments{
\item{compilation}{A single string containing the name of the
Snaptron data source. Any variant of the \code{Compilation} enum can also be
passed an argument.}

\item{regions}{Either a list of 1 more \code{HUGO} gene names as strings
\verb{e.g. "BRCA1"} or a Granges class object containing one or more geneomic
intervals \verb{(e.g. "chr1:1-1000")}.}
}
\value{
A QueryBuilder object.
}
\description{
Construct a QueryBuilder object given a compilation and one or regions.
}
\examples{
# contruct a query builder for GTEX data source and BRAC1 gene
qb <- QueryBuilder(compilation = Compilation$gtex, regions = "BRCA1")

# contruct a query builder for TCGA data source and chromosome region
qb <- QueryBuilder(compilation = "tcga", regions = "chr1:1-1000")

# construct a query builder for TCGA data source using GRanges object
library(GenomicRanges)
qb <- QueryBuilder(compilation = "tcga", regions = GRanges("chr1", "1-1000"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snapcount.R
\docType{package}
\name{snapcount-package}
\alias{snapcount}
\alias{snapcount-package}
\title{snapcount: an R package for interfacing with Snaptron}
\description{
snapcount is a client interface to the Snaptron webservice which supports
querying by gene name or genomic region.
}
\details{
Results include raw expression counts derived from alignment of RNA-seq
samples and/or various summarized measures of expression across one or more
regions/genes per-sample (e.g. percent spliced in).

To learn more about snapcount, check out the vignette:
\code{browseVignettes(package = "snapcount")}
}
\section{Package options}{

\describe{
\item{\code{snapcount.host}}{Change the host that snapcount uses when
connecting to Snaptron. Default: \code{snaptron.cs.jhu.edu}}
\item{\code{snapcount.port}}{Change the port that snapcount uses when
connecting to Snaptron. Default: \code{80}}
}
}

\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/langmead-lab/snapcount}
  \item Report bugs at \url{https://github.com/langmead-lab/snapcount/issues}
}

}
\author{
\strong{Maintainer}: Rone Charles \email{rcharle8@jh.edu}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basic_query_functions.R
\name{query_jx}
\alias{query_jx}
\alias{query_gene}
\alias{query_exon}
\title{Query Junctions/Genes/Exons}
\usage{
query_jx(sb, return_rse = TRUE, split_by_region = FALSE)

query_gene(sb, return_rse = TRUE, split_by_region = FALSE)

query_exon(sb, return_rse = TRUE, split_by_region = FALSE)
}
\arguments{
\item{sb}{A SnaptronQueryBuilder object}

\item{return_rse}{Should the query data be returned as a simple data frame or
converted to a RangedSummarizedExperiment.}

\item{split_by_region}{By default the results from multiple queries will be
returned in a \code{RangedSummarizedExperiment} object with a \code{rowData} entry for
each, labeling each result row according to the query it resulted
from. However, if this is set to \code{TRUE}, the result will be a list of
RangedSummarizedExperiment objects, one per original interval/gene. This
latter option may be useful, but it requires a separate copy of the sample
metadata for each original interval/gene.}
}
\value{
Functions will return either a RangedSummarizedExperiment or
data.table depending on whether the \code{return_rse} parameter is set to
\code{TRUE} or \code{FALSE}.
}
\description{
Given one or more gene names or genomic range
intervals it will return a list of 0 or more genes, junctions, or exons
(depending on which query form is used) which overlap the ranges.
}
\examples{
# Contruct a QueryBuilder object
qb <- QueryBuilder(compilation = "gtex", regions = "chr1:1-100000")
qb <- set_row_filters(qb, samples_count >= 20)
query_jx(qb)

qb <- set_row_filters(qb, NULL)
qb <- set_column_filters(qb, SMTS == "Brain")
query_gene(qb)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higher_level_functions.R
\name{junction_union}
\alias{junction_union}
\title{Get the union of junctions from 2 or more compilations
which are on the same reference}
\usage{
junction_union(...)
}
\arguments{
\item{...}{One or more QueryBuilder objects}
}
\value{
A RangedSummarizedExperiment of junctions appearing in at least
one compilation
}
\description{
This function queries 2 or more compilations which are on the same
reference version (e.g. hg38) and merges the resulting junctions
into a single output table, unioning the sample coverage columns
and the snaptron_id (jx ID) columns (the latter delimiter will
be ":").  All sample IDs will be disjoint between compilations.
}
\details{
Union is based on the following fields (combined into a comparison key):
\itemize{
\item group
\item chromosome
\item start
\item end
\item strand
}

The goal is to have a single list of junctions where every junction
occurs in at least one compilation and if a junction occurs in > 1
compilations it still only has a single row representing all the
samples across compilations that it appears in.
Sample aggregate statistics will be recalculated for junctions which
are merged across \emph{all} samples from all compilations:

\itemize{
\item sample_count
\item coverage_sum
\item coverage_avg
\item coverage_median
}
}
\examples{
# Using query builder wrappers
sb1 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879786-1879786")
sb1 <- set_coordinate_modifier(sb1, Coordinates$EndIsExactOrWithin)
sb1 <- set_row_filters(sb1, strand == "-")

sb2 <- QueryBuilder(compilation = "tcga", regions = "chr1:1879786-1879786")
sb2 <- set_coordinate_modifier(sb2, Coordinates$EndIsExactOrWithin)
sb2 <- set_row_filters(sb2, strand == "-")

junction_union(sb1, sb2)
}
\seealso{
\code{\link[=junction_intersection]{junction_intersection()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higher_level_functions.R
\name{junction_inclusion_ratio}
\alias{junction_inclusion_ratio}
\title{Relative measure of splice variant usage similar to PSI that allows
for 2 arbitrarily defined groups of junctions (not limited to
cassette exons).}
\usage{
junction_inclusion_ratio(group1, group2, group_names = NULL)
}
\arguments{
\item{group1, group2}{Each group is a list of 1 or more
QueryBuilder objects}

\item{group_names}{Optional vector of strings representing the
group names}
}
\value{
A DataFrame of samples, with their JIR score and metadata,
which had > 0 coverage in at least one resulting row in at
least one of the groups
}
\description{
Calculates a coverage summary statistic per sample of the normalized
coverage difference between two sets of separate junctions defined
by at least two basic queries and organized into two groups.
}
\details{
The summary statistic is as follows:
If the coverage of the first group is "A" and the second is "B":

\code{JIR(A,B)=(A - B) / (A+B+1)}

This is calculated for every sample that occurs in one or the other
(or both) groups results.
}
\examples{
sb1 <- QueryBuilder(compilation = "srav2", regions = "chr2:29446395-30142858")
sb1 <- set_coordinate_modifier(sb1, Coordinates$Within)
sb1 <- set_row_filters(sb1, strand == "-")

sb2 <- QueryBuilder(compilation = "srav2", regions = "chr2:29416789-29446394")
sb2 <- set_coordinate_modifier(sb2, Coordinates$Within)
sb2 <- set_row_filters(sb2, strand == "-")

junction_inclusion_ratio(list(sb1), list(sb2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{get_sids}
\alias{get_sids}
\alias{set_sids}
\title{Get or set query sample ids}
\usage{
get_sids(qb)

set_sids(qb, sids)
}
\arguments{
\item{qb}{a QueryBuilder object constructed using the
\code{\link{QueryBuilder}} function.}

\item{sids}{a vector or 1 or more whole numbers to filter results on.}
}
\value{
\code{get_sids} returns the current sample ids as a vector of integers.
\code{set_sids} returns a new \code{QueryBuilder} object with the
sample ids set to the value of \code{sids}.
}
\description{
Get or set query sample ids
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
qb <- set_sids(qb, c(1, 2, 3))
get_sids(qb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enums.R
\docType{data}
\name{Coordinates}
\alias{Coordinates}
\title{Enum for Snaptron Coordinate modifiers}
\format{
An object of class \code{environment} of length 4.
}
\usage{
Coordinates
}
\description{
Enum for Snaptron Coordinate modifiers
}
\section{Fields}{

\describe{
\item{\code{Exact}}{Return junctions whose start and end
coordinates match the boundaries of the
region requested.}

\item{\code{Within}}{Return junctions whose start and end
coordinates are within the boundaries of the
region requested.}

\item{\code{StartIsExactOrWithin}}{Return junctions whose
start coordinate matches, or is within, the
boundaries of the region requested.}

\item{\code{EndIsExactOrWithin}}{Return junctions whose
end coordinate matches, or is within, the
boundaries of the region requested.}
}}

\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
qb <- set_coordinate_modifier(qb, Coordinates$Exact)
qb
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/enums.R
\docType{data}
\name{Compilation}
\alias{Compilation}
\title{Enum for Snaptron compilations}
\format{
An object of class \code{environment} of length 21.
}
\usage{
Compilation
}
\description{
The variants for this enum will be populated dynamically after the
package has been loaded. If the package cannot connect to the internet
the variants will default to:
}
\details{
\itemize{
\item{gtex}
\item{tcga}
\item{srav2}
\item{sra}
}
}
\examples{
qb <- QueryBuilder(compilation = Compilation$gtex, regions = "KCNIP4")
query_jx(qb)
}
\seealso{
\url{http://snaptron.cs.jhu.edu/data.html} for more information
about Snaptron compilations.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{get_row_filters}
\alias{get_row_filters}
\alias{set_row_filters}
\title{Get or set range-related contraints for query}
\usage{
get_row_filters(qb)

set_row_filters(qb, ...)
}
\arguments{
\item{qb}{a QueryBuilder object constructed using the
\code{\link{QueryBuilder}} function.}

\item{...}{one or more boolean predicates as either strings or unevaluated
expressions.}
}
\value{
\code{get_row_filters} returns the current row filters as list of strings.
\code{set_row_filters} returns a new \code{QueryBuilder} object with
the row filters set to the value of \code{row_filters}.
}
\description{
Get or set range-related contraints for query
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
# row filters set as a string
qb <- set_row_filters(qb, "strand == +")
get_row_filters(qb)
# row filters set using unevaluated expression
qb <- set_row_filters(qb, strand == "+")
get_row_filters(qb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higher_level_functions.R
\name{percent_spliced_in}
\alias{percent_spliced_in}
\title{Relative measure of splice variant usage, limited currently to
cassette exon splice variants}
\usage{
percent_spliced_in(
  inclusion_group1,
  inclusion_group2,
  exclusion_group,
  min_count = 20,
  group_names = NULL
)
}
\arguments{
\item{inclusion_group1, inclusion_group2, exclusion_group}{Where each
is a list of 1 or more QueryBuilder objects}

\item{min_count}{minimum total count (denominator) required to not
be assigned -1}

\item{group_names}{Optional vector of strings representing the
group names}
}
\value{
A DataFrame of samples, with their PSI score and metadata,
which had > 0 coverage in at least one resulting row in at
least one of the groups
}
\description{
Similar to the JIR, this calculates Percent Spliced In (PSI) statistics
for the definition of 2 different groups: inclusion and exclusion.
Currently this function only supports the cassette exon use case.
}
\details{
Inclusion typically defines 2 basic queries, one for the junction
preceding the cassette exon, and the second for the junction following
the cassette exon.  The exclusion group contains one basic query
which defines the junction which skips the cassette exon.

The PSI itself is implemented as:

\code{PSI(inclusion1, inclusion2, exclusion) =
  mean(inclusion1, inclusion2) / (mean(inclusion1, inclusion2) + exclusion)}

where each term denotes the coverage of junctions that resulted
from the basic queries in that group in the current sample.
}
\examples{
in1 <- QueryBuilder(compilation = "srav2", regions = "chr1:94468008-94472172")
in1 <- set_coordinate_modifier(in1, Coordinates$Exact)
in1 <- set_row_filters(in1, strand == "+")

in2 <- QueryBuilder(compilation = "srav2", regions = "chr1:94468008-94472172")
in2 <- set_coordinate_modifier(in2, Coordinates$Exact)
in2 <- set_row_filters(in2, strand == "+")

ex <- QueryBuilder(compilation = "srav2", regions = "chr1:94468008-94475142")
ex <- set_coordinate_modifier(ex, Coordinates$Exact)
ex <- set_row_filters(ex, strand == "+")

percent_spliced_in(list(in1), list(in2), list(ex))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higher_level_functions.R
\name{tissue_specificity}
\alias{tissue_specificity}
\title{Tissue Specificity (TS): produces a list of samples with their tissues
marked which either contain queried junctions (1) or not (0); can be used
as input to significance testing methods such as Kruskal-Wallis to look for
tissue enrichment (currently only works for the GTEx compilation).}
\usage{
tissue_specificity(..., group_names = NULL)
}
\arguments{
\item{...}{One or more QueryBuilder objects}

\item{group_names}{Optional vector of strings representing the
group names}
}
\value{
A DataFrame of all samples in the compilation with either a
0 or 1 indicating their occurrence and shared status (if > 1
group passed in).  Occurrence here is if the sample has at least
one result with > 0 coverage, and further, if > 1 group is passed
in, then if it occurs in the results of all groups.  Also includes the
sample tissue type and sample_id.
}
\description{
Lists the number of samples labeled with a specific tissue type.
Samples are filtered for ones which have junctions across all the
user-specified groups. That is, if a sample only appears in the results of
some of the groups (from their basic queries) it will be assigned a 0,
otherwise if it is in all of the groups' results it will be assigned a 1.
This is similar to the SSC high level query type, but doesn't sum the
coverage.
}
\details{
The samples are then grouped by their tissue type (e.g. Brain).
This is useful for determining if there's an enrichment for a specific
tissue in the set of junctions queried.  Results from this can be fed to a
statistical test, such as the Kruskal-wallis non-parametric rank test.
This query is limited to GTEx only, due to the fact that GTEx is one of the
few compilations that has consistent and complete tissue metadata.
}
\examples{
in1 <- QueryBuilder(compilation = "gtex", regions = "chr4:20763023-20763023")
in1 <- set_coordinate_modifier(in1, Coordinates$EndIsExactOrWithin)
in1 <- set_row_filters(in1, strand == "-")

in2 <- QueryBuilder(compilation = "gtex", regions = "chr4:20763098-20763098")
in2 <- set_coordinate_modifier(in2, Coordinates$StartIsExactOrWithin)
in2 <- set_row_filters(in2, strand == "-")

tissue_specificity(list(in1, in2))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{get_coordinate_modifier}
\alias{get_coordinate_modifier}
\alias{set_coordinate_modifier}
\title{Get or set coordinate modifiers for the query.}
\usage{
get_coordinate_modifier(qb)

set_coordinate_modifier(qb, coordinate_modifier)
}
\arguments{
\item{qb}{a QueryBuilder object constructed using the
\code{\link{QueryBuilder}} function.}

\item{coordinate_modifier}{any of the variants of the \link{Coordinates}
enum.}
}
\value{
\code{get_coordinate_modifier} returns the current coodinate modifier
as a string.
\code{set_coordinate_modifier} returns a new \code{QueryBuilder}
object with the coordinate modifier set to the value of
\code{coordinate_modifier}.
}
\description{
Get or set coordinate modifiers for the query.
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
qb <- set_coordinate_modifier(qb, Coordinates$Within)
get_coordinate_modifier(qb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{get_column_filters}
\alias{get_column_filters}
\alias{set_column_filters}
\title{Get or set sample-related contraints for query}
\usage{
get_column_filters(qb)

set_column_filters(qb, ...)
}
\arguments{
\item{qb}{a QueryBuilder object constructed using the
\code{\link{QueryBuilder}} function}

\item{...}{one or more boolean predicates as either strings or unevaluated
expressions}
}
\value{
\code{get_column_filters} returns the current filters as a list of strings.
\code{set_column_filters} returns a new \code{QueryBuilder} object
with the column filters set to the value of \code{column_filters}.
}
\description{
Get or set sample-related contraints for query
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
# column filters set using a string
qb <- set_column_filters(qb, "SMTS == Brain")
get_column_filters(qb)
# column filters set using unevaluated expression
qb <- set_column_filters(qb, SMTS == "Spleen")
get_column_filters(qb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/basic_query_functions.R
\name{uri_of_last_successful_request}
\alias{uri_of_last_successful_request}
\title{Return the URI of the last successful request to Snaptron}
\usage{
uri_of_last_successful_request()
}
\value{
URI of last successful request to Snaptron or \code{NULL} if there
have not been any successful requests.
}
\description{
This function can be paired with the \code{from_url} method from
the QueryBuilder class, allowing users to share sources of data
from Snaptron.
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
query_jx(qb)
uri_of_last_successful_request()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{get_regions}
\alias{get_regions}
\alias{set_regions}
\title{Get or set query regions}
\usage{
get_regions(qb)

set_regions(qb, regions)
}
\arguments{
\item{qb}{A QueryBuilder object constructed using the
\code{\link{QueryBuilder}} function.}

\item{regions}{Either a list of 1 more \code{HUGO} gene names as strings
\verb{e.g. "BRCA1"} or a Granges class object containing one or more geneomic
intervals \verb{(e.g. "chr1:1-1000")}.}
}
\value{
\code{get_regions} returns the current regions as a list of strings.
\code{set_regions} returns a new \code{QueryBuilder} object with the
regions set to the value of \code{regions}.
}
\description{
Get or set query regions
}
\examples{
qb <- QueryBuilder(compilation = "gtex", regions = "CD99")
get_regions(qb)
qb <- set_regions(qb, "chr1:1-1000")
get_regions(qb)
qb <- set_regions(qb, GenomicRanges::GRanges("chr1", "1-1000"))
get_regions(qb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/query_builder_wrappers.R
\name{from_url}
\alias{from_url}
\title{Constructs a \code{QueryBuilder} object from the given url}
\usage{
from_url(url)
}
\arguments{
\item{url}{a well-formed url preferably obtained from a call to the
\code{\link{uri_of_last_successful_request}} function}
}
\value{
Returns a \code{QueryBuilder} object with attributes set from the
parsed url.
}
\description{
Constructs a \code{QueryBuilder} object from the given url
}
\examples{
sb <- from_url("http://snaptron.cs.jhu.edu/gtex/snaptron?regions=CD99")
get_regions(sb)
get_compilation(sb)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higher_level_functions.R
\name{junction_intersection}
\alias{junction_intersection}
\title{Get the intersection of junctions from 2 or more compilations
which are on the same reference}
\usage{
junction_intersection(...)
}
\arguments{
\item{...}{One or more QueryBuilder objects}
}
\value{
A RangedSummarizedExperiment of junctions common across compilations
}
\description{
This function operates similar to the \code{junction_union()} function, i.e
it is cross compilation and merging of the same junction from multiple
compilations will be handled exactly the same way. But instead
of every junction which appears in at least one compilation, only
the junctions which appear in \emph{every} compilation will be returned.
}
\examples{
# Using query builder wrappers
sb1 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879786-1879786")
sb1 <- set_coordinate_modifier(sb1, Coordinates$EndIsExactOrWithin)
sb1 <- set_row_filters(sb1, strand == "-")

sb2 <- QueryBuilder(compilation = "tcga", regions = "chr1:1879786-1879786")
sb2 <- set_coordinate_modifier(sb2, Coordinates$EndIsExactOrWithin)
sb2 <- set_row_filters(sb2, strand == "-")

junction_intersection(sb1, sb2)
}
\seealso{
\code{\link[=junction_union]{junction_union()}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/higher_level_functions.R
\name{shared_sample_counts}
\alias{shared_sample_counts}
\title{Shared Sample Count (SSC): counts total number of samples in which 2
different junctions both occur in.}
\usage{
shared_sample_counts(..., group_names = NULL)
}
\arguments{
\item{...}{One or more lists of QueryBuilder objects}

\item{group_names}{Optional vector of character strings
representing group names}
}
\value{
A DataFrame of results based on the list of groups passed
in via "group_names".  Each group is reported with the # of
unique samples which occurred in all of its defined set of
related basic queries (e.g. two inclusion basic queries in a
cassette exon scenario).
}
\description{
This produces a list of user-specified groups and the read coverage
of the junctions in all the samples which were shared across all
the basic queries occurring in each group.
}
\details{
Example: User defines a single group of junctions "GroupA" made up of 2
separate regions (two basic queries).

An SSC query will return a single line for GroupA which will have
the total number of samples which had at least one junction which
was returned from both basic queries. It will also report a summary
statistic of the total number of groups which had one or more
samples that were shared across the basic queries, in this case it
would be 1.  Also, it will report the number of groups which had at
least one shared sample and which had matching junctions (from the
query) which were fully annotated.

This function can be used to determine how much cross-sample
support there is for a particular junction configuration (typically
a cassette exon).
}
\examples{
g1 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879786-1879786")
g1 <- set_coordinate_modifier(g1, Coordinates$EndIsExactOrWithin)
g1 <- set_row_filters(g1, strand == "-")

g2 <- QueryBuilder(compilation = "gtex", regions = "chr1:1879903-1879903")
g2 <- set_coordinate_modifier(g2, Coordinates$StartIsExactOrWithin)
g2 <- set_row_filters(g2, strand == "-")

ssc<-shared_sample_counts(list(g1, g2))
}
